@@ -43,6 +43,9 @@ The current mod is the working directory, or the directory specified by the --mo
 	}
 
 	cmdconfig.OnCmd(cmd).
+		AddCloudFlags().
+		AddWorkspaceDatabaseFlag().
+		AddModLocationFlag().
 		AddBoolFlag(constants.ArgHelp, false, "Help for dashboard", cmdconfig.FlagOptions.WithShortHand("h")).
 		AddBoolFlag(constants.ArgModInstall, true, "Specify whether to install mod dependencies before running the dashboard").
 		AddStringFlag(constants.ArgDashboardListen, string(dashboardserver.ListenTypeLocal), "Accept connections from: local (localhost only) or network (open)").
@@ -58,9 +61,9 @@ The current mod is the working directory, or the directory specified by the --mo
 		AddStringArrayFlag(constants.ArgVariable, nil, "Specify the value of a variable").
 		AddBoolFlag(constants.ArgInput, true, "Enable interactive prompts").
 		AddStringFlag(constants.ArgOutput, constants.OutputFormatNone, "Select a console output format: none, snapshot").
-		AddBoolFlag(constants.ArgSnapshot, false, "Create snapshot in Steampipe Cloud with the default (workspace) visibility").
-		AddBoolFlag(constants.ArgShare, false, "Create snapshot in Steampipe Cloud with 'anyone_with_link' visibility").
-		AddStringFlag(constants.ArgSnapshotLocation, "", "The location to write snapshots - either a local file path or a Steampipe Cloud workspace").
+		AddBoolFlag(constants.ArgSnapshot, false, "Create snapshot in Turbot Pipes with the default (workspace) visibility").
+		AddBoolFlag(constants.ArgShare, false, "Create snapshot in Turbot Pipes with 'anyone_with_link' visibility").
+		AddStringFlag(constants.ArgSnapshotLocation, "", "The location to write snapshots - either a local file path or a Turbot Pipes workspace").
 		AddStringFlag(constants.ArgSnapshotTitle, "", "The title to give a snapshot").
 		// NOTE: use StringArrayFlag for ArgDashboardInput, not StringSliceFlag
 		// Cobra will interpret values passed to a StringSliceFlag as CSV, where args passed to StringArrayFlag are not parsed and used raw
@@ -97,7 +100,7 @@ func runDashboardCmd(cmd *cobra.Command, args []string) {
 	error_helpers.FailOnError(err)
 
 	// if diagnostic mode is set, print out config and return
-	if _, ok := os.LookupEnv(constants.EnvDiagnostics); ok {
+	if _, ok := os.LookupEnv(constants.EnvConfigDump); ok {
 		cmdconfig.DisplayConfig()
 		return
 	}
@@ -121,7 +124,11 @@ func runDashboardCmd(cmd *cobra.Command, args []string) {
 	serverListen := dashboardserver.ListenType(viper.GetString(constants.ArgDashboardListen))
 	error_helpers.FailOnError(serverListen.IsValid())
 
-	if err := utils.IsPortBindable(int(serverPort)); err != nil {
+	serverHost := ""
+	if serverListen == dashboardserver.ListenTypeLocal {
+		serverHost = "127.0.0.1"
+	}
+	if err := utils.IsPortBindable(serverHost, int(serverPort)); err != nil {
 		exitCode = constants.ExitCodeBindPortUnavailable
 		error_helpers.FailOnError(err)
 	}
@@ -241,7 +248,7 @@ func initDashboard(ctx context.Context) *initialisation.InitData {
 func getInitData(ctx context.Context) *initialisation.InitData {
 	w, errAndWarnings := workspace.LoadWorkspacePromptingForVariables(ctx)
 	if errAndWarnings.GetError() != nil {
-		return initialisation.NewErrorInitData(fmt.Errorf("failed to load workspace: %s", errAndWarnings.GetError().Error()))
+		return initialisation.NewErrorInitData(fmt.Errorf("failed to load workspace: %s", error_helpers.HandleCancelError(errAndWarnings.GetError()).Error()))
 	}
 
 	i := initialisation.NewInitData()
@@ -431,11 +438,11 @@ func saveDashboardState(serverPort dashboardserver.ListenPort, serverListen dash
 		Pid:        os.Getpid(),
 		Port:       int(serverPort),
 		ListenType: string(serverListen),
-		Listen:     constants.DatabaseListenAddresses,
+		Listen:     constants.DashboardListenAddresses,
 	}
 
 	if serverListen == dashboardserver.ListenTypeNetwork {
-		addrs, _ := utils.LocalAddresses()
+		addrs, _ := utils.LocalPublicAddresses()
 		state.Listen = append(state.Listen, addrs...)
 	}
 	error_helpers.FailOnError(dashboardserver.WriteServiceStateFile(state))
