@@ -56,6 +56,9 @@ Examples:
 	cmd.AddCommand(modInitCmd())
 	cmd.Flags().BoolP(constants.ArgHelp, "h", false, "Help for mod")
 
+	cmdconfig.OnCmd(cmd).
+		AddModLocationFlag()
+
 	return cmd
 }
 
@@ -65,14 +68,35 @@ func modInstallCmd() *cobra.Command {
 		Use:   "install",
 		Run:   runModInstallCmd,
 		Short: "Install one or more mods and their dependencies",
-		Long:  `Install one or more mods and their dependencies.`,
+		Long: `Install one or more mods and their dependencies.
+
+Mods provide an easy way to share Steampipe queries, controls, and benchmarks.
+Find mods using the public registry at hub.steampipe.io.
+
+Examples:
+
+  # Install a mod(steampipe-mod-aws-compliance)
+  steampipe mod install github.com/turbot/steampipe-mod-aws-compliance
+
+  # Install a specific version of a mod
+  steampipe mod install github.com/turbot/steampipe-mod-aws-compliance@0.1
+
+  # Install a version of a mod using a semver constraint
+  steampipe mod install github.com/turbot/steampipe-mod-aws-compliance@'^1'
+
+  # Install all mods specified in the mod.sp and their dependencies
+  steampipe mod install
+
+  # Preview what steampipe mod install will do, without actually installing anything
+  steampipe mod install --dry-run`,
 	}
 
 	cmdconfig.OnCmd(cmd).
 		AddBoolFlag(constants.ArgPrune, true, "Remove unused dependencies after installation is complete").
 		AddBoolFlag(constants.ArgDryRun, false, "Show which mods would be installed/updated/uninstalled without modifying them").
 		AddBoolFlag(constants.ArgForce, false, "Install mods even if plugin/cli version requirements are not met (cannot be used with --dry-run)").
-		AddBoolFlag(constants.ArgHelp, false, "Help for install", cmdconfig.FlagOptions.WithShortHand("h"))
+		AddBoolFlag(constants.ArgHelp, false, "Help for install", cmdconfig.FlagOptions.WithShortHand("h")).
+		AddModLocationFlag()
 
 	return cmd
 }
@@ -121,13 +145,19 @@ func modUninstallCmd() *cobra.Command {
 		Use:   "uninstall",
 		Run:   runModUninstallCmd,
 		Short: "Uninstall a mod and its dependencies",
-		Long:  `Uninstall a mod and its dependencies.`,
+		Long: `Uninstall a mod and its dependencies.
+
+Example:
+  
+  # Uninstall a mod
+  steampipe mod uninstall github.com/turbot/steampipe-mod-azure-compliance`,
 	}
 
 	cmdconfig.OnCmd(cmd).
 		AddBoolFlag(constants.ArgPrune, true, "Remove unused dependencies after uninstallation is complete").
 		AddBoolFlag(constants.ArgDryRun, false, "Show which mods would be uninstalled without modifying them").
-		AddBoolFlag(constants.ArgHelp, false, "Help for uninstall", cmdconfig.FlagOptions.WithShortHand("h"))
+		AddBoolFlag(constants.ArgHelp, false, "Help for uninstall", cmdconfig.FlagOptions.WithShortHand("h")).
+		AddModLocationFlag()
 
 	return cmd
 }
@@ -165,14 +195,23 @@ func modUpdateCmd() *cobra.Command {
 		Use:   "update",
 		Run:   runModUpdateCmd,
 		Short: "Update one or more mods and their dependencies",
-		Long:  `Update one or more mods and their dependencies.`,
+		Long: `Update one or more mods and their dependencies.
+
+Example:
+
+  # Update a mod to the latest version allowed by its current constraint
+  steampipe mod update github.com/turbot/steampipe-mod-aws-compliance
+
+  # Update all mods specified in the mod.sp and their dependencies to the latest versions that meet their constraints, and install any that are missing
+  steampipe mod update`,
 	}
 
 	cmdconfig.OnCmd(cmd).
 		AddBoolFlag(constants.ArgPrune, true, "Remove unused dependencies after update is complete").
 		AddBoolFlag(constants.ArgForce, false, "Update mods even if plugin/cli version requirements are not met (cannot be used with --dry-run)").
 		AddBoolFlag(constants.ArgDryRun, false, "Show which mods would be updated without modifying them").
-		AddBoolFlag(constants.ArgHelp, false, "Help for update", cmdconfig.FlagOptions.WithShortHand("h"))
+		AddBoolFlag(constants.ArgHelp, false, "Help for update", cmdconfig.FlagOptions.WithShortHand("h")).
+		AddModLocationFlag()
 
 	return cmd
 }
@@ -211,10 +250,17 @@ func modListCmd() *cobra.Command {
 		Use:   "list",
 		Run:   runModListCmd,
 		Short: "List currently installed mods",
-		Long:  `List currently installed mods.`,
+		Long: `List currently installed mods.
+		
+Example:
+
+  # List installed mods
+  steampipe mod list`,
 	}
 
-	cmdconfig.OnCmd(cmd).AddBoolFlag(constants.ArgHelp, false, "Help for list", cmdconfig.FlagOptions.WithShortHand("h"))
+	cmdconfig.OnCmd(cmd).
+		AddBoolFlag(constants.ArgHelp, false, "Help for list", cmdconfig.FlagOptions.WithShortHand("h")).
+		AddModLocationFlag()
 	return cmd
 }
 
@@ -255,10 +301,17 @@ func modInitCmd() *cobra.Command {
 		Use:   "init",
 		Run:   runModInitCmd,
 		Short: "Initialize the current directory with a mod.sp file",
-		Long:  `Initialize the current directory with a mod.sp file.`,
+		Long: `Initialize the current directory with a mod.sp file.
+		
+Example:
+
+  # Initialize the current directory with a mod.sp file
+  steampipe mod init`,
 	}
 
-	cmdconfig.OnCmd(cmd).AddBoolFlag(constants.ArgHelp, false, "Help for init", cmdconfig.FlagOptions.WithShortHand("h"))
+	cmdconfig.OnCmd(cmd).
+		AddBoolFlag(constants.ArgHelp, false, "Help for init", cmdconfig.FlagOptions.WithShortHand("h")).
+		AddModLocationFlag()
 	return cmd
 }
 
@@ -278,12 +331,15 @@ func runModInitCmd(cmd *cobra.Command, args []string) {
 		exitCode = constants.ExitCodeModInitFailed
 		error_helpers.FailOnError(err)
 	}
-	fmt.Printf("Created mod definition file '%s'\n", filepaths.ModFilePath(workspacePath))
 }
 
 // helpers
 func createWorkspaceMod(ctx context.Context, cmd *cobra.Command, workspacePath string) (*modconfig.Mod, error) {
-	if !modinstaller.ValidateModLocation(ctx, workspacePath) {
+	cancel, err := modinstaller.ValidateModLocation(ctx, workspacePath)
+	if err != nil {
+		return nil, err
+	}
+	if !cancel {
 		return nil, fmt.Errorf("mod %s cancelled", cmd.Name())
 	}
 
@@ -291,14 +347,19 @@ func createWorkspaceMod(ctx context.Context, cmd *cobra.Command, workspacePath s
 		fmt.Println("Working folder already contains a mod definition file")
 		return nil, nil
 	}
+	// write mod definition file
 	mod := modconfig.CreateDefaultMod(workspacePath)
 	if err := mod.Save(); err != nil {
 		return nil, err
 	}
+	// only print message for mod init (not for mod install)
+	if cmd.Name() == "init" {
+		fmt.Printf("Created mod definition file '%s'\n", filepaths.ModFilePath(workspacePath))
+	}
 
 	// load up the written mod file so that we get the updated
 	// block ranges
-	mod, err := parse.LoadModfile(workspacePath)
+	mod, err = parse.LoadModfile(workspacePath)
 	if err != nil {
 		return nil, err
 	}
