@@ -2,6 +2,8 @@ package cmd
 
 import (
 	"context"
+	"encoding/json"
+	"errors"
 	"fmt"
 	"strings"
 	"sync"
@@ -11,6 +13,7 @@ import (
 	"github.com/spf13/cobra"
 	"github.com/spf13/viper"
 	"github.com/turbot/go-kit/helpers"
+	"github.com/turbot/steampipe-plugin-sdk/v5/sperr"
 	"github.com/turbot/steampipe/pkg/cmdconfig"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/contexthelpers"
@@ -27,6 +30,24 @@ import (
 	"github.com/turbot/steampipe/pkg/utils"
 )
 
+type installedPlugin struct {
+	Name        string   `json:"name"`
+	Version     string   `json:"version"`
+	Connections []string `json:"connections"`
+}
+
+type failedPlugin struct {
+	Name        string   `json:"name"`
+	Reason      string   `json:"reason"`
+	Connections []string `json:"connections"`
+}
+
+type pluginJsonOutput struct {
+	Installed []installedPlugin `json:"installed"`
+	Failed    []failedPlugin    `json:"failed"`
+	Warnings  []string          `json:"warnings"`
+}
+
 // Plugin management commands
 func pluginCmd() *cobra.Command {
 	var cmd = &cobra.Command{
@@ -51,8 +72,12 @@ Examples:
 
   # Uninstall a plugin
   steampipe plugin uninstall aws`,
+		PersistentPostRun: func(_ *cobra.Command, args []string) {
+			utils.LogTime("cmd.plugin.PersistentPostRun start")
+			defer utils.LogTime("cmd.plugin.PersistentPostRun end")
+			plugin.CleanupOldTmpDirs()
+		},
 	}
-
 	cmd.AddCommand(pluginInstallCmd())
 	cmd.AddCommand(pluginListCmd())
 	cmd.AddCommand(pluginUninstallCmd())
@@ -78,6 +103,9 @@ is latest. The name is a required argument.
 
 Examples:
 
+  # Install all missing plugins that are specified in configuration files
+  steampipe plugin install
+
   # Install a common plugin (turbot/aws)
   steampipe plugin install aws
 
@@ -85,12 +113,16 @@ Examples:
   steampipe plugin install turbot/azure@0.1.0
 
   # Hide progress bars during installation
-  steampipe plugin install --progress=false aws`,
+  steampipe plugin install --progress=false aws
+
+  # Skip creation of default plugin config file
+  steampipe plugin install --skip-config aws`,
 	}
 
 	cmdconfig.
 		OnCmd(cmd).
 		AddBoolFlag(constants.ArgProgress, true, "Display installation progress").
+		AddBoolFlag(constants.ArgSkipConfig, false, "Skip creating the default config file for plugin").
 		AddBoolFlag(constants.ArgHelp, false, "Help for plugin install", cmdconfig.FlagOptions.WithShortHand("h"))
 	return cmd
 }
@@ -111,7 +143,7 @@ is latest. The name is a required argument.
 
 Examples:
 
-  # Update all plugins to their latest available version 
+  # Update all plugins to their latest available version
   steampipe plugin update --all
 
   # Update a common plugin (turbot/aws)
@@ -147,14 +179,17 @@ Examples:
   steampipe plugin list
 
   # List plugins that have updates available
-  steampipe plugin list --outdated`,
+  steampipe plugin list --outdated
+
+  # List plugins output in json
+  steampipe plugin list --output json`,
 	}
 
 	cmdconfig.
 		OnCmd(cmd).
 		AddBoolFlag("outdated", false, "Check each plugin in the list for updates").
+		AddStringFlag(constants.ArgOutput, "table", "Output format: table or json").
 		AddBoolFlag(constants.ArgHelp, false, "Help for plugin list", cmdconfig.FlagOptions.WithShortHand("h"))
-
 	return cmd
 }
 
@@ -213,13 +248,17 @@ func runPluginInstallCmd(cmd *cobra.Command, args []string) {
 	installReports := make(display.PluginInstallReports, 0, len(plugins))
 
 	if len(plugins) == 0 {
-		fmt.Println()
-		error_helpers.ShowError(ctx, fmt.Errorf("you need to provide at least one plugin to install"))
-		fmt.Println()
-		cmd.Help()
-		fmt.Println()
-		exitCode = constants.ExitCodeInsufficientOrWrongInputs
-		return
+		if len(steampipeconfig.GlobalConfig.Plugins) == 0 {
+			error_helpers.ShowError(ctx, sperr.New("No connections or plugins configured"))
+			exitCode = constants.ExitCodeInsufficientOrWrongInputs
+			return
+		}
+
+		// get the list of plugins to install
+		for imageRef := range steampipeconfig.GlobalConfig.Plugins {
+			ref := ociinstaller.NewSteampipeImageRef(imageRef)
+			plugins = append(plugins, ref.GetFriendlyName())
+		}
 	}
 
 	// a leading blank line - since we always output multiple lines
@@ -279,6 +318,7 @@ func doPluginInstall(ctx context.Context, bar *uiprogress.Bar, pluginName string
 	pluginAlreadyInstalled, _ := plugin.Exists(pluginName)
 	if pluginAlreadyInstalled {
 		// set the bar to MAX
+		//nolint:golint,errcheck // the error happens if we set this over the max value
 		bar.Set(len(pluginInstallSteps))
 		// let the bar append itself with "Already Installed"
 		bar.AppendFunc(func(b *uiprogress.Bar) string {
@@ -381,7 +421,13 @@ func runPluginUpdateCmd(cmd *cobra.Command, args []string) {
 			ref := ociinstaller.NewSteampipeImageRef(p)
 			isExists, _ := plugin.Exists(p)
 			if isExists {
-				runUpdatesFor = append(runUpdatesFor, versionData.Plugins[ref.DisplayImageRef()])
+				if strings.HasPrefix(ref.DisplayImageRef(), constants.SteampipeHubOCIBase) {
+					runUpdatesFor = append(runUpdatesFor, versionData.Plugins[ref.DisplayImageRef()])
+				} else {
+					error_helpers.ShowError(ctx, fmt.Errorf("cannot check updates for plugins not distributed via hub.steampipe.io, you should uninstall then reinstall the plugin to get the latest version"))
+					exitCode = constants.ExitCodePluginLoadingError
+					return
+				}
 			} else {
 				exitCode = constants.ExitCodePluginNotFound
 				updateResults = append(updateResults, &display.PluginInstallReport{
@@ -508,7 +554,7 @@ func installPlugin(ctx context.Context, pluginName string, isUpdate bool, bar *u
 		}
 	}()
 
-	image, err := plugin.Install(ctx, pluginName, progress)
+	image, err := plugin.Install(ctx, pluginName, progress, ociinstaller.WithSkipConfig(viper.GetBool(constants.ArgSkipConfig)))
 	if err != nil {
 		msg := ""
 		_, name, stream := ociinstaller.NewSteampipeImageRef(pluginName).GetOrgNameAndStream()
@@ -561,10 +607,11 @@ func resolveUpdatePluginsFromArgs(args []string) ([]string, error) {
 	return plugins, nil
 }
 
-func runPluginListCmd(cmd *cobra.Command, args []string) {
+func runPluginListCmd(cmd *cobra.Command, _ []string) {
 	// setup a cancel context and start cancel handler
 	ctx, cancel := context.WithCancel(cmd.Context())
 	contexthelpers.StartCancelHandler(cancel)
+	outputFormat := viper.GetString(constants.ArgOutput)
 
 	utils.LogTime("runPluginListCmd list")
 	defer func() {
@@ -582,21 +629,41 @@ func runPluginListCmd(cmd *cobra.Command, args []string) {
 		return
 	}
 
+	err := showPluginListOutput(pluginList, failedPluginMap, missingPluginMap, res, outputFormat)
+	if err != nil {
+		error_helpers.ShowError(cmd.Context(), err)
+	}
+
+}
+
+func showPluginListOutput(pluginList []plugin.PluginListItem, failedPluginMap, missingPluginMap map[string][]*modconfig.Connection, res *error_helpers.ErrorAndWarnings, outputFormat string) error {
+	switch outputFormat {
+	case "table":
+		return showPluginListAsTable(pluginList, failedPluginMap, missingPluginMap, res)
+	case "json":
+		return showPluginListAsJSON(pluginList, failedPluginMap, missingPluginMap, res)
+	default:
+		return errors.New("invalid output format")
+	}
+}
+
+func showPluginListAsTable(pluginList []plugin.PluginListItem, failedPluginMap, missingPluginMap map[string][]*modconfig.Connection, res *error_helpers.ErrorAndWarnings) error {
+	headers := []string{"Installed", "Version", "Connections"}
+	var rows [][]string
 	// List installed plugins in a table
 	if len(pluginList) != 0 {
-		headers := []string{"Installed Plugin", "Version", "Connections"}
-		var rows [][]string
 		for _, item := range pluginList {
-			rows = append(rows, []string{item.Name, item.Version, strings.Join(item.Connections, ",")})
+			rows = append(rows, []string{item.Name, item.Version.String(), strings.Join(item.Connections, ",")})
 		}
-		display.ShowWrappedTable(headers, rows, &display.ShowWrappedTableOptions{AutoMerge: false})
-		fmt.Printf("\n")
+	} else {
+		rows = append(rows, []string{"", "", ""})
 	}
+	display.ShowWrappedTable(headers, rows, &display.ShowWrappedTableOptions{AutoMerge: false})
+	fmt.Printf("\n")
 
 	// List failed/missing plugins in a separate table
 	if len(failedPluginMap)+len(missingPluginMap) != 0 {
-		// List missing/failed plugins
-		headers := []string{"Failed Plugin", "Connections", "Reason"}
+		headers := []string{"Failed", "Connections", "Reason"}
 		var conns []string
 		var missingRows [][]string
 
@@ -608,6 +675,7 @@ func runPluginListCmd(cmd *cobra.Command, args []string) {
 			missingRows = append(missingRows, []string{p, strings.Join(conns, ","), constants.ConnectionErrorPluginFailedToStart})
 			conns = []string{}
 		}
+
 		// missing plugins
 		for p, item := range missingPluginMap {
 			for _, conn := range item {
@@ -616,7 +684,9 @@ func runPluginListCmd(cmd *cobra.Command, args []string) {
 			missingRows = append(missingRows, []string{p, strings.Join(conns, ","), constants.InstallMessagePluginNotInstalled})
 			conns = []string{}
 		}
+
 		display.ShowWrappedTable(headers, missingRows, &display.ShowWrappedTableOptions{AutoMerge: false})
+		fmt.Println()
 	}
 
 	if len(res.Warnings) > 0 {
@@ -624,38 +694,58 @@ func runPluginListCmd(cmd *cobra.Command, args []string) {
 		res.ShowWarnings()
 		fmt.Printf("\n")
 	}
+	return nil
 }
 
-func getPluginList(ctx context.Context) (pluginList []plugin.PluginListItem, failedPluginMap, missingPluginMap map[string][]*modconfig.Connection, res *modconfig.ErrorAndWarnings) {
-	statushooks.Show(ctx)
-	defer statushooks.Done(ctx)
+func showPluginListAsJSON(pluginList []plugin.PluginListItem, failedPluginMap, missingPluginMap map[string][]*modconfig.Connection, res *error_helpers.ErrorAndWarnings) error {
+	output := pluginJsonOutput{}
 
-	// get the maps of available and failed/missing plugins
-	pluginConnectionMap, failedPluginMap, missingPluginMap, res := getPluginConnectionMap(ctx)
-	if res.Error != nil {
-		return nil, nil, nil, res
-	}
-
-	// get a list of the installed plugins by inspecting the install location
-	// pass pluginConnectionMap so we can populate the connections for each plugin
-	// we need to look at installed plugins - we can't just use the plugin connection map
-	// because the plugin connection map only contains entries for plugins which have connections
-	pluginList, err := plugin.List(pluginConnectionMap)
-	if err != nil {
-		res.Error = err
-		return nil, nil, nil, res
-	}
-
-	// remove the failed plugins from `list` since we don't want them in the installed table
-	for pluginName := range failedPluginMap {
-		for i := 0; i < len(pluginList); i++ {
-			if pluginList[i].Name == pluginName {
-				pluginList = append(pluginList[:i], pluginList[i+1:]...)
-				i-- // Decrement the loop index since we just removed an element
-			}
+	for _, item := range pluginList {
+		installed := installedPlugin{
+			Name:        item.Name,
+			Version:     item.Version.String(),
+			Connections: item.Connections,
 		}
+		output.Installed = append(output.Installed, installed)
 	}
-	return pluginList, failedPluginMap, missingPluginMap, res
+
+	for p, item := range failedPluginMap {
+		connections := make([]string, len(item))
+		for i, conn := range item {
+			connections[i] = conn.Name
+		}
+		failed := failedPlugin{
+			Name:        p,
+			Connections: connections,
+			Reason:      constants.ConnectionErrorPluginFailedToStart,
+		}
+		output.Failed = append(output.Failed, failed)
+	}
+
+	for p, item := range missingPluginMap {
+		connections := make([]string, len(item))
+		for i, conn := range item {
+			connections[i] = conn.Name
+		}
+		missing := failedPlugin{
+			Name:        p,
+			Connections: connections,
+			Reason:      constants.InstallMessagePluginNotInstalled,
+		}
+		output.Failed = append(output.Failed, missing)
+	}
+
+	if len(res.Warnings) > 0 {
+		output.Warnings = res.Warnings
+	}
+
+	jsonOutput, err := json.MarshalIndent(output, "", "  ")
+	if err != nil {
+		return err
+	}
+	fmt.Println(string(jsonOutput))
+	fmt.Println()
+	return nil
 }
 
 func runPluginUninstallCmd(cmd *cobra.Command, args []string) {
@@ -690,7 +780,7 @@ func runPluginUninstallCmd(cmd *cobra.Command, args []string) {
 		return
 	}
 
-	reports := display.PluginRemoveReports{}
+	reports := steampipeconfig.PluginRemoveReports{}
 	statushooks.SetStatus(ctx, fmt.Sprintf("Uninstalling %s", utils.Pluralize("plugin", len(args))))
 	for _, p := range args {
 		statushooks.SetStatus(ctx, fmt.Sprintf("Uninstalling %s", p))
@@ -708,10 +798,44 @@ func runPluginUninstallCmd(cmd *cobra.Command, args []string) {
 	reports.Print()
 }
 
-func getPluginConnectionMap(ctx context.Context) (pluginConnectionMap, failedPluginMap, missingPluginMap map[string][]*modconfig.Connection, res *modconfig.ErrorAndWarnings) {
+func getPluginList(ctx context.Context) (pluginList []plugin.PluginListItem, failedPluginMap, missingPluginMap map[string][]*modconfig.Connection, res *error_helpers.ErrorAndWarnings) {
+	statushooks.Show(ctx)
+	defer statushooks.Done(ctx)
+
+	// get the maps of available and failed/missing plugins
+	pluginConnectionMap, failedPluginMap, missingPluginMap, res := getPluginConnectionMap(ctx)
+	if res.Error != nil {
+		return nil, nil, nil, res
+	}
+
+	// TODO do we really need to look at installed plugins - can't we just use the plugin connection map
+	// get a list of the installed plugins by inspecting the install location
+	// pass pluginConnectionMap so we can populate the connections for each plugin
+	pluginList, err := plugin.List(pluginConnectionMap)
+	if err != nil {
+		res.Error = err
+		return nil, nil, nil, res
+	}
+
+	// remove the failed plugins from `list` since we don't want them in the installed table
+	for pluginName := range failedPluginMap {
+		for i := 0; i < len(pluginList); i++ {
+			if pluginList[i].Name == pluginName {
+				pluginList = append(pluginList[:i], pluginList[i+1:]...)
+				i-- // Decrement the loop index since we just removed an element
+			}
+		}
+	}
+	return pluginList, failedPluginMap, missingPluginMap, res
+}
+
+func getPluginConnectionMap(ctx context.Context) (pluginConnectionMap, failedPluginMap, missingPluginMap map[string][]*modconfig.Connection, res *error_helpers.ErrorAndWarnings) {
+	utils.LogTime("cmd.getPluginConnectionMap start")
+	defer utils.LogTime("cmd.getPluginConnectionMap end")
+
 	statushooks.SetStatus(ctx, "Fetching connection map")
 
-	res = &modconfig.ErrorAndWarnings{}
+	res = &error_helpers.ErrorAndWarnings{}
 
 	connectionStateMap, stateRes := getConnectionState(ctx)
 	res.Merge(stateRes)
@@ -743,7 +867,10 @@ func getPluginConnectionMap(ctx context.Context) (pluginConnectionMap, failedPlu
 }
 
 // load the connection state, waiting until all connections are loaded
-func getConnectionState(ctx context.Context) (steampipeconfig.ConnectionStateMap, *modconfig.ErrorAndWarnings) {
+func getConnectionState(ctx context.Context) (steampipeconfig.ConnectionStateMap, *error_helpers.ErrorAndWarnings) {
+	utils.LogTime("cmd.getConnectionState start")
+	defer utils.LogTime("cmd.getConnectionState end")
+
 	// start service
 	client, res := db_local.GetLocalClient(ctx, constants.InvokerPlugin, nil)
 	if res.Error != nil {
@@ -751,7 +878,7 @@ func getConnectionState(ctx context.Context) (steampipeconfig.ConnectionStateMap
 	}
 	defer client.Close(ctx)
 
-	conn, err := client.AcquireConnection(ctx)
+	conn, err := client.AcquireManagementConnection(ctx)
 	if err != nil {
 		res.Error = err
 		return nil, res
