@@ -5,20 +5,20 @@ import (
 	"log"
 	"os"
 	"os/signal"
-	"path/filepath"
 	"strings"
 	"syscall"
 	"time"
 
 	"github.com/hashicorp/go-hclog"
 	"github.com/spf13/cobra"
+	"github.com/turbot/go-kit/helpers"
+	"github.com/turbot/go-kit/logging"
 	"github.com/turbot/go-kit/types"
-	"github.com/turbot/steampipe-plugin-sdk/v5/logging"
+	sdklogging "github.com/turbot/steampipe-plugin-sdk/v5/logging"
+	"github.com/turbot/steampipe-plugin-sdk/v5/plugin"
 	"github.com/turbot/steampipe/pkg/cmdconfig"
 	"github.com/turbot/steampipe/pkg/connection"
 	"github.com/turbot/steampipe/pkg/constants"
-	"github.com/turbot/steampipe/pkg/constants/runtime"
-	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/filepaths"
 	"github.com/turbot/steampipe/pkg/pluginmanager_service"
 	"github.com/turbot/steampipe/pkg/steampipeconfig"
@@ -35,7 +35,44 @@ func pluginManagerCmd() *cobra.Command {
 }
 
 func runPluginManagerCmd(cmd *cobra.Command, _ []string) {
-	ctx := cmd.Context()
+	var err error
+	defer func() {
+		if r := recover(); r != nil {
+			err = helpers.ToError(r)
+		}
+		if err != nil {
+			// write to stdout so the plugin manager can extract the error message
+			fmt.Println(fmt.Sprintf("%s%s", plugin.PluginStartupFailureMessage, err.Error()))
+		}
+		os.Exit(1)
+	}()
+
+	err = doRunPluginManager(cmd)
+}
+
+func doRunPluginManager(cmd *cobra.Command) error {
+	pluginManager, err := createPluginManager(cmd)
+	if err != nil {
+		return err
+	}
+
+	if shouldRunConnectionWatcher() {
+		log.Printf("[INFO] starting connection watcher")
+		connectionWatcher, err := connection.NewConnectionWatcher(pluginManager)
+		if err != nil {
+			return err
+		}
+
+		// close the connection watcher
+		defer connectionWatcher.Close()
+	}
+
+	log.Printf("[INFO] about to serve")
+	pluginManager.Serve()
+	return nil
+}
+
+func createPluginManager(cmd *cobra.Command) (*pluginmanager_service.PluginManager, error) {
 	logger := createPluginManagerLog()
 
 	log.Printf("[INFO] starting plugin manager")
@@ -43,7 +80,7 @@ func runPluginManagerCmd(cmd *cobra.Command, _ []string) {
 	steampipeConfig, errorsAndWarnings := steampipeconfig.LoadConnectionConfig()
 	if errorsAndWarnings.GetError() != nil {
 		log.Printf("[WARN] failed to load connection config: %v", errorsAndWarnings.GetError())
-		os.Exit(1)
+		return nil, errorsAndWarnings.Error
 	}
 
 	// add signal handler for sigpipe - this will be raised if we call displayWarning as stdout is piped
@@ -56,37 +93,17 @@ func runPluginManagerCmd(cmd *cobra.Command, _ []string) {
 		}
 	}()
 
-	// add a prefix to the PgClientAppName so that out DB connecitons are not treated as
-	// another Steampipe instance connected to the DB
-	// (as our lifecycle is managed by the db service,
-	// so we will be shut down when the service is stopped)
-	runtime.PgClientAppName = runtime.PgClientAppNamePluginManagerPrefix + runtime.PgClientAppName
-
+	// create a map of connections configs, excluding connections in error
 	configMap := connection.NewConnectionConfigMap(steampipeConfig.Connections)
 	log.Printf("[TRACE] loaded config map: %s", strings.Join(steampipeConfig.ConnectionNames(), ","))
 
-	pluginManager, err := pluginmanager_service.NewPluginManager(configMap, logger)
+	pluginManager, err := pluginmanager_service.NewPluginManager(cmd.Context(), configMap, steampipeConfig.PluginsInstances, logger)
 	if err != nil {
 		log.Printf("[WARN] failed to create plugin manager: %s", err.Error())
-		os.Exit(1)
+		return nil, err
 	}
 
-	if shouldRunConnectionWatcher() {
-		log.Printf("[INFO] starting connection watcher")
-
-		connectionWatcher, err := connection.NewConnectionWatcher(pluginManager.OnConnectionConfigChanged)
-		if err != nil {
-			log.Printf("[WARN] failed to create connection watcher: %s", err.Error())
-			error_helpers.ShowError(ctx, err)
-			os.Exit(1)
-		}
-
-		// close the connection watcher
-		defer connectionWatcher.Close()
-	}
-
-	log.Printf("[TRACE] about to serve")
-	pluginManager.Serve()
+	return pluginManager, nil
 }
 
 func shouldRunConnectionWatcher() bool {
@@ -100,15 +117,15 @@ func shouldRunConnectionWatcher() bool {
 }
 
 func createPluginManagerLog() hclog.Logger {
-	logName := fmt.Sprintf("plugin-%s.log", time.Now().Format("2006-01-02"))
-	logPath := filepath.Join(filepaths.EnsureLogDir(), logName)
-	f, err := os.OpenFile(logPath, os.O_RDWR|os.O_CREATE|os.O_APPEND, 0666)
-	if err != nil {
-		fmt.Printf("failed to open plugin manager log file: %s\n", err.Error())
-		os.Exit(3)
-	}
-	logger := logging.NewLogger(&hclog.LoggerOptions{
-		Output:     f,
+	// we use this logger to log from the plugin processes
+	// the plugin processes uses the `EscapeNewlineWriter` to map the '\n' byte to "\n" string literal
+	// this is to allow the plugin to send multiline log messages as a single log line.
+	//
+	// here we apply the reverse mapping to get back the original message
+	writer := sdklogging.NewUnescapeNewlineWriter(logging.NewRotatingLogWriter(filepaths.EnsureLogDir(), "plugin"))
+
+	logger := sdklogging.NewLogger(&hclog.LoggerOptions{
+		Output:     writer,
 		TimeFn:     func() time.Time { return time.Now().UTC() },
 		TimeFormat: "2006-01-02 15:04:05.000 UTC",
 	})
