@@ -5,7 +5,6 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"github.com/turbot/steampipe/pkg/connection_sync"
 	"os"
 	"path"
 	"strings"
@@ -14,7 +13,9 @@ import (
 	"github.com/spf13/cobra"
 	"github.com/spf13/viper"
 	"github.com/turbot/go-kit/helpers"
+	"github.com/turbot/steampipe-plugin-sdk/v5/sperr"
 	"github.com/turbot/steampipe/pkg/cmdconfig"
+	"github.com/turbot/steampipe/pkg/connection_sync"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/contexthelpers"
 	"github.com/turbot/steampipe/pkg/dashboard/dashboardexecute"
@@ -71,6 +72,9 @@ Examples:
 	// * In the future we may add --csv and --json flags as shortcuts for --output
 	cmdconfig.
 		OnCmd(cmd).
+		AddCloudFlags().
+		AddWorkspaceDatabaseFlag().
+		AddModLocationFlag().
 		AddBoolFlag(constants.ArgHelp, false, "Help for query", cmdconfig.FlagOptions.WithShortHand("h")).
 		AddBoolFlag(constants.ArgHeader, true, "Include column headers csv and table output").
 		AddStringFlag(constants.ArgSeparator, ",", "Separator string for csv output").
@@ -85,13 +89,13 @@ Examples:
 		// where args passed to StringArrayFlag are not parsed and used raw
 		AddStringArrayFlag(constants.ArgVariable, nil, "Specify the value of a variable").
 		AddBoolFlag(constants.ArgInput, true, "Enable interactive prompts").
-		AddBoolFlag(constants.ArgSnapshot, false, "Create snapshot in Steampipe Cloud with the default (workspace) visibility").
-		AddBoolFlag(constants.ArgShare, false, "Create snapshot in Steampipe Cloud with 'anyone_with_link' visibility").
+		AddBoolFlag(constants.ArgSnapshot, false, "Create snapshot in Turbot Pipes with the default (workspace) visibility").
+		AddBoolFlag(constants.ArgShare, false, "Create snapshot in Turbot Pipes with 'anyone_with_link' visibility").
 		AddStringArrayFlag(constants.ArgSnapshotTag, nil, "Specify tags to set on the snapshot").
 		AddStringFlag(constants.ArgSnapshotTitle, "", "The title to give a snapshot").
 		AddIntFlag(constants.ArgDatabaseQueryTimeout, 0, "The query timeout").
 		AddStringSliceFlag(constants.ArgExport, nil, "Export output to file, supported format: sps (snapshot)").
-		AddStringFlag(constants.ArgSnapshotLocation, "", "The location to write snapshots - either a local file path or a Steampipe Cloud workspace").
+		AddStringFlag(constants.ArgSnapshotLocation, "", "The location to write snapshots - either a local file path or a Turbot Pipes workspace").
 		AddBoolFlag(constants.ArgProgress, true, "Display snapshot upload status")
 
 	cmd.AddCommand(getListSubCmd(listSubCmdOptions{parentCmd: cmd}))
@@ -109,27 +113,34 @@ func runQueryCmd(cmd *cobra.Command, args []string) {
 		}
 	}()
 
-	if stdinData := getPipedStdinData(); len(stdinData) > 0 {
-		args = append(args, stdinData)
-	}
-
 	// validate args
 	err := validateQueryArgs(ctx, args)
 	error_helpers.FailOnError(err)
 
 	// if diagnostic mode is set, print out config and return
-	if _, ok := os.LookupEnv(constants.EnvDiagnostics); ok {
+	if _, ok := os.LookupEnv(constants.EnvConfigDump); ok {
 		cmdconfig.DisplayConfig()
 		return
 	}
 
+	if len(args) == 0 {
+		// no positional arguments - check if there's anything on stdin
+		if stdinData := getPipedStdinData(); len(stdinData) > 0 {
+			// we have data - treat this as an argument
+			args = append(args, stdinData)
+		}
+	}
+
 	// enable paging only in interactive mode
 	interactiveMode := len(args) == 0
 	// set config to indicate whether we are running an interactive query
 	viper.Set(constants.ConfigKeyInteractive, interactiveMode)
 
+	// initialize the cancel handler - for context cancellation
+	initCtx, cancel := context.WithCancel(ctx)
+	contexthelpers.StartCancelHandler(cancel)
 	// start the initializer
-	initData := query.NewInitData(ctx, args)
+	initData := query.NewInitData(initCtx, args)
 	if initData.Result.Error != nil {
 		exitCode = constants.ExitCodeInitializationFailed
 		error_helpers.ShowError(ctx, initData.Result.Error)
@@ -166,11 +177,11 @@ func validateQueryArgs(ctx context.Context, args []string) error {
 	interactiveMode := len(args) == 0
 	if interactiveMode && (viper.IsSet(constants.ArgSnapshot) || viper.IsSet(constants.ArgShare)) {
 		exitCode = constants.ExitCodeInsufficientOrWrongInputs
-		return fmt.Errorf("cannot share snapshots in interactive mode")
+		return sperr.New("cannot share snapshots in interactive mode")
 	}
 	if interactiveMode && len(viper.GetStringSlice(constants.ArgExport)) > 0 {
 		exitCode = constants.ExitCodeInsufficientOrWrongInputs
-		return fmt.Errorf("cannot export query results in interactive mode")
+		return sperr.New("cannot export query results in interactive mode")
 	}
 	// if share or snapshot args are set, there must be a query specified
 	err := cmdconfig.ValidateSnapshotArgs(ctx)
@@ -183,7 +194,7 @@ func validateQueryArgs(ctx context.Context, args []string) error {
 	output := viper.GetString(constants.ArgOutput)
 	if !helpers.StringSliceContains(validOutputFormats, output) {
 		exitCode = constants.ExitCodeInsufficientOrWrongInputs
-		return fmt.Errorf("invalid output format: '%s', must be one of [%s]", output, strings.Join(validOutputFormats, ", "))
+		return sperr.New("invalid output format: '%s', must be one of [%s]", output, strings.Join(validOutputFormats, ", "))
 	}
 
 	return nil
@@ -279,11 +290,11 @@ func snapshotToQueryResult(snap *dashboardtypes.SteampipeSnapshot) (*queryresult
 	// the table of a snapshot query has a fixed name
 	tablePanel, ok := snap.Panels[modconfig.SnapshotQueryTableName]
 	if !ok {
-		return nil, fmt.Errorf("dashboard does not contain table result for query")
+		return nil, sperr.New("dashboard does not contain table result for query")
 	}
 	chartRun := tablePanel.(*dashboardexecute.LeafRun)
 	if !ok {
-		return nil, fmt.Errorf("failed to read query result from snapshot")
+		return nil, sperr.New("failed to read query result from snapshot")
 	}
 	// check for error
 	if err := chartRun.GetError(); err != nil {
