@@ -4,6 +4,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"log"
 	"os"
 	"os/signal"
 	"strings"
@@ -13,13 +14,16 @@ import (
 	"github.com/spf13/cobra"
 	"github.com/spf13/viper"
 	"github.com/turbot/go-kit/helpers"
+	"github.com/turbot/steampipe-plugin-sdk/v5/sperr"
 	"github.com/turbot/steampipe/pkg/cmdconfig"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/dashboard/dashboardserver"
 	"github.com/turbot/steampipe/pkg/db/db_local"
 	"github.com/turbot/steampipe/pkg/display"
 	"github.com/turbot/steampipe/pkg/error_helpers"
+	"github.com/turbot/steampipe/pkg/filepaths"
 	"github.com/turbot/steampipe/pkg/pluginmanager"
+	pb "github.com/turbot/steampipe/pkg/pluginmanager_service/grpc/proto"
 	"github.com/turbot/steampipe/pkg/statushooks"
 	"github.com/turbot/steampipe/pkg/utils"
 )
@@ -58,11 +62,10 @@ connection from any Postgres compatible database client.`,
 
 	cmdconfig.
 		OnCmd(cmd).
+		AddModLocationFlag().
 		AddBoolFlag(constants.ArgHelp, false, "Help for service start", cmdconfig.FlagOptions.WithShortHand("h")).
-		// for now default port to -1 so we fall back to the default of the deprecated arg
 		AddIntFlag(constants.ArgDatabasePort, constants.DatabaseDefaultPort, "Database service port").
-		// for now default listen address to empty so we fall back to the default of the deprecated arg
-		AddStringFlag(constants.ArgListenAddress, string(db_local.ListenTypeNetwork), "Accept connections from: local (localhost only) or network (open) (postgres)").
+		AddStringFlag(constants.ArgDatabaseListenAddresses, string(db_local.ListenTypeNetwork), "Accept connections from: `local` (an alias for `localhost` only), `network` (an alias for `*`), or a comma separated list of hosts and/or IP addresses").
 		AddStringFlag(constants.ArgServicePassword, "", "Set the database password for this session").
 		// default is false and hides the database user password from service start prompt
 		AddBoolFlag(constants.ArgServiceShowPassword, false, "View database password for connecting from another machine").
@@ -162,25 +165,21 @@ func runServiceStartCmd(cmd *cobra.Command, _ []string) {
 	ctx, cancel := signal.NotifyContext(cmd.Context(), os.Interrupt, os.Kill)
 	defer cancel()
 
+	listenAddresses := db_local.StartListenType(viper.GetString(constants.ArgDatabaseListenAddresses)).ToListenAddresses()
+
 	port := viper.GetInt(constants.ArgDatabasePort)
 	if port < 1 || port > 65535 {
 		exitCode = constants.ExitCodeInsufficientOrWrongInputs
 		panic("Invalid port - must be within range (1:65535)")
 	}
 
-	serviceListen := db_local.StartListenType(viper.GetString(constants.ArgListenAddress))
-	if serviceListen.IsValid() != nil {
-		exitCode = constants.ExitCodeInsufficientOrWrongInputs
-		error_helpers.FailOnError(serviceListen.IsValid())
-	}
-
 	invoker := constants.Invoker(cmdconfig.Viper().GetString(constants.ArgInvoker))
 	if invoker.IsValid() != nil {
 		exitCode = constants.ExitCodeInsufficientOrWrongInputs
 		error_helpers.FailOnError(invoker.IsValid())
 	}
 
-	startResult, dashboardState, dbServiceStarted := startService(ctx, port, serviceListen, invoker)
+	startResult, dashboardState, dbServiceStarted := startService(ctx, listenAddresses, port, invoker)
 	alreadyRunning := !dbServiceStarted
 
 	printStatus(ctx, startResult.DbState, startResult.PluginManagerState, dashboardState, alreadyRunning)
@@ -190,9 +189,10 @@ func runServiceStartCmd(cmd *cobra.Command, _ []string) {
 	}
 }
 
-func startService(ctx context.Context, port int, serviceListen db_local.StartListenType, invoker constants.Invoker) (_ *db_local.StartResult, _ *dashboardserver.DashboardServiceState, dbServiceStarted bool) {
+func startService(ctx context.Context, listenAddresses []string, port int, invoker constants.Invoker) (_ *db_local.StartResult, _ *dashboardserver.DashboardServiceState, dbServiceStarted bool) {
 	statushooks.Show(ctx)
 	defer statushooks.Done(ctx)
+	log.Printf("[TRACE] startService - listenAddresses=%q", listenAddresses)
 
 	err := db_local.EnsureDBInstalled(ctx)
 	if err != nil {
@@ -201,29 +201,25 @@ func startService(ctx context.Context, port int, serviceListen db_local.StartLis
 	}
 
 	// start db, refreshing connections
-	startResult := db_local.StartServices(ctx, port, serviceListen, invoker)
-	if startResult.Error != nil {
-		exitCode = constants.ExitCodeServiceSetupFailure
-		error_helpers.FailOnError(startResult.Error)
-	}
-
+	startResult := startServiceAndRefreshConnections(ctx, listenAddresses, port, invoker)
 	if startResult.Status == db_local.ServiceFailedToStart {
-		error_helpers.ShowError(ctx, fmt.Errorf("steampipe service failed to start"))
+		error_helpers.ShowError(ctx, sperr.New("steampipe service failed to start"))
 		exitCode = constants.ExitCodeServiceStartupFailure
 		return
 	}
 
 	// if the service is already running, then service start should make the service persistent
 	if startResult.Status == db_local.ServiceAlreadyRunning {
-
 		// check that we have the same port and listen parameters
 		if port != startResult.DbState.Port {
 			exitCode = constants.ExitCodeInsufficientOrWrongInputs
-			error_helpers.FailOnError(fmt.Errorf("service is already running on port %d - cannot change port while it's running", startResult.DbState.Port))
+			error_helpers.FailOnError(sperr.New("service is already running on port %d - cannot change port while it's running", startResult.DbState.Port))
 		}
-		if serviceListen != startResult.DbState.ListenType {
+		if !startResult.DbState.MatchWithGivenListenAddresses(listenAddresses) {
 			exitCode = constants.ExitCodeInsufficientOrWrongInputs
-			error_helpers.FailOnError(fmt.Errorf("service is already running and listening on %s - cannot change listen type while it's running", startResult.DbState.ListenType))
+			// this messaging assumes that the resolved addresses from the given addresses have not changed while the service is running
+			// although this is an edge case, ideally, we should check for the resolved addresses and give the relevant message
+			error_helpers.FailOnError(sperr.New("service is already running and listening on %s - cannot change listen address while it's running", strings.Join(startResult.DbState.ResolvedListenAddresses, ", ")))
 		}
 
 		// convert to being invoked by service
@@ -258,6 +254,24 @@ func startService(ctx context.Context, port int, serviceListen db_local.StartLis
 	return startResult, dashboardState, dbServiceStarted
 }
 
+func startServiceAndRefreshConnections(ctx context.Context, listenAddresses []string, port int, invoker constants.Invoker) *db_local.StartResult {
+	startResult := db_local.StartServices(ctx, listenAddresses, port, invoker)
+	if startResult.Error != nil {
+		exitCode = constants.ExitCodeServiceStartupFailure
+		error_helpers.FailOnError(startResult.Error)
+	}
+
+	if startResult.Status == db_local.ServiceStarted {
+		// ask the plugin manager to refresh connections
+		// this is executed asyncronously by the plugin manager
+		// we ignore this error, since RefreshConnections is async and all errors will flow through
+		// the notification system
+		// we do not expect any I/O errors on this since the PluginManager is running in the same box
+		_, _ = startResult.PluginManager.RefreshConnections(&pb.RefreshConnectionsRequest{})
+	}
+	return startResult
+}
+
 func tryToStopServices(ctx context.Context) {
 	// stop db service
 	if _, err := db_local.StopServices(ctx, false, constants.InvokerService); err != nil {
@@ -434,8 +448,7 @@ to force a restart.
 	viper.Set(constants.ArgServicePassword, currentDbState.Password)
 
 	// start db
-	dbStartResult := db_local.StartServices(ctx, currentDbState.Port, currentDbState.ListenType, currentDbState.Invoker)
-	error_helpers.FailOnError(dbStartResult.Error)
+	dbStartResult := startServiceAndRefreshConnections(ctx, currentDbState.ResolvedListenAddresses, currentDbState.Port, currentDbState.Invoker)
 	if dbStartResult.Status == db_local.ServiceFailedToStart {
 		exitCode = constants.ExitCodeServiceStartupFailure
 		fmt.Println("Steampipe service was stopped, but failed to restart.")
@@ -465,15 +478,16 @@ func runServiceStatusCmd(cmd *cobra.Command, _ []string) {
 		}
 	}()
 
-	if !db_local.IsInstalled() {
+	if !db_local.IsDBInstalled() || !db_local.IsFDWInstalled() {
 		fmt.Println("Steampipe service is not installed.")
 		return
 	}
+
 	if viper.GetBool(constants.ArgAll) {
 		showAllStatus(cmd.Context())
 	} else {
 		dbState, dbStateErr := db_local.GetState()
-		pmState, pmStateErr := pluginmanager.LoadPluginManagerState()
+		pmState, pmStateErr := pluginmanager.LoadState()
 		dashboardState, dashboardStateErr := dashboardserver.GetDashboardServiceState()
 
 		if dbStateErr != nil || pmStateErr != nil {
@@ -632,7 +646,7 @@ func showAllStatus(ctx context.Context) {
 
 func getServiceProcessDetails(process *psutils.Process) (string, string, string, db_local.StartListenType) {
 	cmdLine, _ := process.CmdlineSlice()
-	installDir := strings.TrimSuffix(cmdLine[0], db_local.ServiceExecutableRelativeLocation())
+	installDir := strings.TrimSuffix(cmdLine[0], filepaths.ServiceExecutableRelativeLocation())
 	var port string
 	var listenType db_local.StartListenType
 
@@ -652,7 +666,7 @@ func getServiceProcessDetails(process *psutils.Process) (string, string, string,
 	return fmt.Sprintf("%d", process.Pid), installDir, port, listenType
 }
 
-func printStatus(ctx context.Context, dbState *db_local.RunningDBInstanceInfo, pmState *pluginmanager.PluginManagerState, dashboardState *dashboardserver.DashboardServiceState, alreadyRunning bool) {
+func printStatus(ctx context.Context, dbState *db_local.RunningDBInstanceInfo, pmState *pluginmanager.State, dashboardState *dashboardserver.DashboardServiceState, alreadyRunning bool) {
 	if dbState == nil && !pmState.Running {
 		fmt.Println("Service is not running")
 		return
@@ -671,13 +685,13 @@ Managing the Steampipe service:
 
   # Get status of the service
   steampipe service status
-	 
+
   # View database password for connecting from another machine
   steampipe service status --show-password
-  
+
   # Restart the service
   steampipe service restart
-  
+
   # Stop the service
   steampipe service stop
 `
@@ -689,7 +703,7 @@ Managing the Steampipe service:
 			"postgres://%v:%v@%v:%v/%v",
 			dbState.User,
 			dbState.Password,
-			dbState.Listen[0],
+			utils.GetFirstListenAddress(dbState.ResolvedListenAddresses),
 			dbState.Port,
 			dbState.Database,
 		)
@@ -698,7 +712,7 @@ Managing the Steampipe service:
 		connectionStr = fmt.Sprintf(
 			"postgres://%v@%v:%v/%v",
 			dbState.User,
-			dbState.Listen[0],
+			utils.GetFirstListenAddress(dbState.ResolvedListenAddresses),
 			dbState.Port,
 			dbState.Database,
 		)
@@ -717,7 +731,7 @@ Database:
 `
 	postgresMsg := fmt.Sprintf(
 		postgresFmt,
-		strings.Join(dbState.Listen, ", "),
+		strings.Join(dbState.ResolvedListenAddresses, ", "),
 		dbState.Port,
 		dbState.Database,
 		dbState.User,
@@ -728,7 +742,7 @@ Database:
 	dashboardMsg := ""
 
 	if dashboardState != nil {
-		browserUrl := fmt.Sprintf("http://localhost:%d/", dashboardState.Port)
+		browserUrl := fmt.Sprintf("http://%s:%d/", dashboardState.Listen[0], dashboardState.Port)
 		dashboardMsg = fmt.Sprintf(`
 Dashboard:
 
@@ -767,7 +781,7 @@ To keep the service running after the %s session completes, use %s.
 		// the service is running, but the plugin_manager is not running and there's no state file
 		// meaning that it cannot be restarted by the FDW
 		// it's an ERROR
-		error_helpers.ShowError(ctx, fmt.Errorf(`
+		error_helpers.ShowError(ctx, sperr.New(`
 Service is running, but the Plugin Manager cannot be recovered.
 Please use %s to recover the service
 `,
