@@ -19,8 +19,8 @@ func Viper() *viper.Viper {
 	return viper.GetViper()
 }
 
-// BootstrapViper sets up viper with the essential path config (workspace-chdir and install-dir)
-func BootstrapViper(loader *steampipeconfig.WorkspaceProfileLoader, cmd *cobra.Command) error {
+// bootstrapViper sets up viper with the essential path config (workspace-chdir and install-dir)
+func bootstrapViper(loader *steampipeconfig.WorkspaceProfileLoader, cmd *cobra.Command) error {
 	// set defaults  for keys which do not have a corresponding command flag
 	setBaseDefaults()
 
@@ -85,6 +85,9 @@ func SetDefaultsFromConfig(configMap map[string]interface{}) {
 // for keys which do not have a corresponding command flag, we need a separate defaulting mechanism
 // any option setting, workspace profile property or env var which does not have a command line
 // MUST have a default (unless we want the zero value to take effect)
+//
+// Do not add keys here which have command line defaults - the way this is setup, this value takes
+// precedence over command line default
 func setBaseDefaults() {
 	defaults := map[string]interface{}{
 		// global general options
@@ -100,7 +103,13 @@ func setBaseDefaults() {
 		constants.ArgDatabaseStartTimeout: constants.DBStartTimeout.Seconds(),
 		constants.ArgServiceCacheEnabled:  true,
 		constants.ArgCacheMaxTtl:          300,
-		constants.ArgMaxCacheSizeMb:       constants.DefaultMaxCacheSizeMb,
+
+		// dashboard
+		constants.ArgDashboardStartTimeout: constants.DashboardStartTimeout.Seconds(),
+
+		// memory
+		constants.ArgMemoryMaxMbPlugin: 1024,
+		constants.ArgMemoryMaxMb:       1024,
 	}
 
 	for k, v := range defaults {
@@ -126,30 +135,40 @@ func setDirectoryDefaultsFromEnv() {
 	}
 }
 
-// set default values from env vars
-func SetDefaultsFromEnv() {
+// setDefaultsFromEnv sets default values from env vars
+func setDefaultsFromEnv() {
 	// NOTE: EnvWorkspaceProfile has already been set as a viper default as we have already loaded workspace profiles
 	// (EnvInstallDir has already been set at same time but we set it again to make sure it has the correct precedence)
 
 	// a map of known environment variables to map to viper keys
 	envMappings := map[string]envMapping{
-		constants.EnvInstallDir:           {[]string{constants.ArgInstallDir}, String},
-		constants.EnvWorkspaceChDir:       {[]string{constants.ArgModLocation}, String},
-		constants.EnvModLocation:          {[]string{constants.ArgModLocation}, String},
-		constants.EnvIntrospection:        {[]string{constants.ArgIntrospection}, String},
-		constants.EnvTelemetry:            {[]string{constants.ArgTelemetry}, String},
-		constants.EnvUpdateCheck:          {[]string{constants.ArgUpdateCheck}, Bool},
-		constants.EnvCloudHost:            {[]string{constants.ArgCloudHost}, String},
-		constants.EnvCloudToken:           {[]string{constants.ArgCloudToken}, String},
-		constants.EnvSnapshotLocation:     {[]string{constants.ArgSnapshotLocation}, String},
-		constants.EnvWorkspaceDatabase:    {[]string{constants.ArgWorkspaceDatabase}, String},
-		constants.EnvServicePassword:      {[]string{constants.ArgServicePassword}, String},
-		constants.EnvCheckDisplayWidth:    {[]string{constants.ArgCheckDisplayWidth}, Int},
-		constants.EnvMaxParallel:          {[]string{constants.ArgMaxParallel}, Int},
-		constants.EnvQueryTimeout:         {[]string{constants.ArgDatabaseQueryTimeout}, Int},
-		constants.EnvDatabaseStartTimeout: {[]string{constants.ArgDatabaseStartTimeout}, Int},
-		constants.EnvCacheTTL:             {[]string{constants.ArgCacheTtl}, Int},
-		constants.EnvCacheMaxTTL:          {[]string{constants.ArgCacheMaxTtl}, Int},
+		constants.EnvInstallDir:     {[]string{constants.ArgInstallDir}, String},
+		constants.EnvWorkspaceChDir: {[]string{constants.ArgModLocation}, String},
+		constants.EnvModLocation:    {[]string{constants.ArgModLocation}, String},
+		constants.EnvIntrospection:  {[]string{constants.ArgIntrospection}, String},
+		constants.EnvTelemetry:      {[]string{constants.ArgTelemetry}, String},
+		constants.EnvUpdateCheck:    {[]string{constants.ArgUpdateCheck}, Bool},
+		// PIPES_HOST needs to be defined before STEAMPIPE_CLOUD_HOST,
+		// so that if STEAMPIPE_CLOUD_HOST is defined, it can override PIPES_HOST
+		constants.EnvPipesHost: {[]string{constants.ArgCloudHost}, String},
+		constants.EnvCloudHost: {[]string{constants.ArgCloudHost}, String},
+		// PIPES_TOKEN needs to be defined before STEAMPIPE_CLOUD_TOKEN,
+		// so that if STEAMPIPE_CLOUD_TOKEN is defined, it can override PIPES_TOKEN
+		constants.EnvPipesToken: {[]string{constants.ArgCloudToken}, String},
+		constants.EnvCloudToken: {[]string{constants.ArgCloudToken}, String},
+		//
+		constants.EnvSnapshotLocation:      {[]string{constants.ArgSnapshotLocation}, String},
+		constants.EnvWorkspaceDatabase:     {[]string{constants.ArgWorkspaceDatabase}, String},
+		constants.EnvServicePassword:       {[]string{constants.ArgServicePassword}, String},
+		constants.EnvDisplayWidth:          {[]string{constants.ArgDisplayWidth}, Int},
+		constants.EnvMaxParallel:           {[]string{constants.ArgMaxParallel}, Int},
+		constants.EnvQueryTimeout:          {[]string{constants.ArgDatabaseQueryTimeout}, Int},
+		constants.EnvDatabaseStartTimeout:  {[]string{constants.ArgDatabaseStartTimeout}, Int},
+		constants.EnvDashboardStartTimeout: {[]string{constants.ArgDashboardStartTimeout}, Int},
+		constants.EnvCacheTTL:              {[]string{constants.ArgCacheTtl}, Int},
+		constants.EnvCacheMaxTTL:           {[]string{constants.ArgCacheMaxTtl}, Int},
+		constants.EnvMemoryMaxMb:           {[]string{constants.ArgMemoryMaxMb}, Int},
+		constants.EnvMemoryMaxMbPlugin:     {[]string{constants.ArgMemoryMaxMbPlugin}, Int},
 
 		// we need this value to go into different locations
 		constants.EnvCacheEnabled: {[]string{
