@@ -15,14 +15,14 @@ import (
 	"github.com/spf13/viper"
 	"github.com/turbot/go-kit/helpers"
 	"github.com/turbot/steampipe-plugin-sdk/v5/plugin"
-	"github.com/turbot/steampipe/pkg/connection_state"
+	"github.com/turbot/steampipe-plugin-sdk/v5/sperr"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/db/db_common"
 	"github.com/turbot/steampipe/pkg/db/db_local"
 	"github.com/turbot/steampipe/pkg/error_helpers"
+	"github.com/turbot/steampipe/pkg/introspection"
 	"github.com/turbot/steampipe/pkg/steampipeconfig"
 	"github.com/turbot/steampipe/pkg/utils"
-	"github.com/turbot/steampipe/sperr"
 	"golang.org/x/exp/maps"
 	"golang.org/x/sync/semaphore"
 )
@@ -33,13 +33,13 @@ type connectionError struct {
 }
 
 type refreshConnectionState struct {
+	// a connection pool to the DB service which uses the server appname
 	pool                       *pgxpool.Pool
 	searchPath                 []string
 	connectionUpdates          *steampipeconfig.ConnectionUpdates
 	tableUpdater               *connectionStateTableUpdater
 	res                        *steampipeconfig.RefreshConnectionResult
 	forceUpdateConnectionNames []string
-
 	// properties for schema/comment cloning
 	exemplarSchemaMapMut sync.Mutex
 
@@ -48,54 +48,62 @@ type refreshConnectionState struct {
 	exemplarSchemaMap map[string]string
 	// if a plugin has an entry in this map, all connections schemas can be cloned from teh exemplar schema
 	exemplarCommentsMap map[string]string
+	pluginManager       pluginManager
 }
 
-func newRefreshConnectionState(ctx context.Context, forceUpdateConnectionNames []string) (*refreshConnectionState, error) {
-	// create a connection pool to connection refresh
-	poolsize := 20
-	pool, err := db_local.CreateConnectionPool(ctx, &db_local.CreateDbOptions{Username: constants.DatabaseSuperUser}, poolsize)
-	if err != nil {
-		return nil, err
-	}
+func newRefreshConnectionState(ctx context.Context, pluginManager pluginManager, forceUpdateConnectionNames []string) (*refreshConnectionState, error) {
+	log.Println("[DEBUG] newRefreshConnectionState start")
+	defer log.Println("[DEBUG] newRefreshConnectionState end")
 
+	pool := pluginManager.Pool()
 	// set user search path first
 	log.Printf("[INFO] setting up search path")
 	searchPath, err := db_local.SetUserSearchPath(ctx, pool)
 	if err != nil {
-		// note: close pool in case of error
-		pool.Close()
 		return nil, err
 	}
 
-	return &refreshConnectionState{
+	res := &refreshConnectionState{
 		pool:                       pool,
 		searchPath:                 searchPath,
 		forceUpdateConnectionNames: forceUpdateConnectionNames,
-	}, nil
-}
-
-func (s *refreshConnectionState) close() {
-	if s.pool != nil {
-		s.pool.Close()
+		pluginManager:              pluginManager,
 	}
+
+	return res, nil
 }
 
 // RefreshConnections loads required connections from config
 // and update the database schema and search path to reflect the required connections
 // return whether any changes have been made
 func (s *refreshConnectionState) refreshConnections(ctx context.Context) {
+	log.Println("[DEBUG] refreshConnectionState.refreshConnections start")
+	defer log.Println("[DEBUG] refreshConnectionState.refreshConnections end")
 	// if there was an error (other than a connection error, which will NOT have been assigned to res),
 	// set state of all incomplete connections to error
 	defer func() {
-		if s.res != nil && s.res.Error != nil {
-			s.setIncompleteConnectionStateToError(ctx, sperr.WrapWithMessage(s.res.Error, "refreshConnections failed before connection update was complete"))
-			// TODO send error PG notification
+		if s.res != nil {
+			if s.res.Error != nil {
+				s.setIncompleteConnectionStateToError(ctx, sperr.WrapWithMessage(s.res.Error, "refreshConnections failed before connection update was complete"))
+			}
+			if !s.res.ErrorAndWarnings.Empty() {
+				log.Printf("[INFO] refreshConnections completed with errors, sending notification")
+				s.pluginManager.SendPostgresErrorsAndWarningsNotification(ctx, &s.res.ErrorAndWarnings)
+			}
+
 		}
 	}()
 	log.Printf("[INFO] building connectionUpdates")
 
-	// determine any necessary connection updates
-	s.connectionUpdates, s.res = steampipeconfig.NewConnectionUpdates(ctx, s.pool, s.forceUpdateConnectionNames...)
+	var opts []steampipeconfig.ConnectionUpdatesOption
+	if len(s.forceUpdateConnectionNames) > 0 {
+		opts = append(opts, steampipeconfig.WithForceUpdate(s.forceUpdateConnectionNames))
+	}
+
+	// build a ConnectionUpdates struct
+	// this determines any necessary connection updates and starts any necessary plugins
+	s.connectionUpdates, s.res = steampipeconfig.NewConnectionUpdates(ctx, s.pool, s.pluginManager, opts...)
+
 	defer s.logRefreshConnectionResults()
 	// were we successful?
 	if s.res.Error != nil {
@@ -104,6 +112,23 @@ func (s *refreshConnectionState) refreshConnections(ctx context.Context) {
 
 	log.Printf("[INFO] created connectionUpdates")
 
+	//  reload plugin rate limiter definitions for all plugins which are updated - the plugin will already be loaded
+	if len(s.connectionUpdates.PluginsWithUpdatedBinary) > 0 {
+		updatedPluginLimiters, err := s.pluginManager.LoadPluginRateLimiters(s.connectionUpdates.PluginsWithUpdatedBinary)
+
+		if err != nil {
+			s.res.Error = err
+			return
+		}
+
+		if len(updatedPluginLimiters) > 0 {
+			err := s.pluginManager.HandlePluginLimiterChanges(updatedPluginLimiters)
+			if err != nil {
+				s.pluginManager.SendPostgresErrorsAndWarningsNotification(ctx, error_helpers.NewErrorsAndWarning(err))
+			}
+		}
+	}
+
 	// delete the connection state file - it will be rewritten when we are complete
 	log.Printf("[INFO] deleting connections state file")
 	steampipeconfig.DeleteConnectionStateFile()
@@ -122,6 +147,7 @@ func (s *refreshConnectionState) refreshConnections(ctx context.Context) {
 
 	// NOTE: delete any DYNAMIC plugin connections which will be updated
 	// to avoid them being accessed before they are updated
+	// TODO sure we can remove this
 	s.executeDeleteQueries(ctx, s.connectionUpdates.DynamicUpdates())
 
 	// update connectionState table to reflect the updates (i.e. set connections to updating/deleting/ready as appropriate)
@@ -152,19 +178,22 @@ func (s *refreshConnectionState) refreshConnections(ctx context.Context) {
 func (s *refreshConnectionState) addMissingPluginWarnings() {
 	log.Printf("[INFO] refreshConnections: identify missing plugins")
 
-	var connectionNames, pluginNames []string
+	var connectionNames []string
 	// add warning if there are connections left over, from missing plugins
 	if len(s.connectionUpdates.MissingPlugins) > 0 {
 		// warning
-		for a, conns := range s.connectionUpdates.MissingPlugins {
+		for _, conns := range s.connectionUpdates.MissingPlugins {
 			for _, con := range conns {
 				connectionNames = append(connectionNames, con.Name)
 			}
-			pluginNames = append(pluginNames, utils.GetPluginName(a))
+
 		}
-		s.res.AddWarning(fmt.Sprintf("%d %s required by %s %s missing. To install, please run %s",
+		pluginNames := maps.Keys(s.connectionUpdates.MissingPlugins)
+
+		s.res.AddWarning(fmt.Sprintf("%d %s required by %d %s %s missing. To install, please run: %s",
 			len(pluginNames),
 			utils.Pluralize("plugin", len(pluginNames)),
+			len(connectionNames),
 			utils.Pluralize("connection", len(connectionNames)),
 			utils.Pluralize("is", len(pluginNames)),
 			constants.Bold(fmt.Sprintf("steampipe plugin install %s", strings.Join(pluginNames, " ")))))
@@ -189,47 +218,45 @@ func (s *refreshConnectionState) logRefreshConnectionResults() {
 }
 
 func (s *refreshConnectionState) executeConnectionQueries(ctx context.Context) {
-	// retrieve updates from the table updater
-	connectionUpdates := s.tableUpdater.updates
-
-	utils.LogTime("db.executeConnectionQueries start")
-	defer utils.LogTime("db.executeConnectionQueries start")
+	log.Println("[DEBUG] refreshConnectionState.executeConnectionQueries start")
+	defer log.Println("[DEBUG] refreshConnectionState.executeConnectionQueries end")
 
 	// execute deletions
-	if err := s.executeDeleteQueries(ctx, maps.Keys(s.connectionUpdates.Delete)); err != nil {
+	if err := s.executeDeleteQueries(ctx, s.connectionUpdates.GetConnectionsToDelete()); err != nil {
 		// just log
 		log.Printf("[WARN] failed to delete all unused schemas: %s", err.Error())
 	}
 
 	// execute updates
-	numUpdates := len(connectionUpdates.Update)
-	numMissingComments := len(connectionUpdates.MissingComments)
+	numUpdates := len(s.connectionUpdates.Update)
+	numMissingComments := len(s.connectionUpdates.MissingComments)
 	log.Printf("[INFO] executeConnectionQueries: num updates: %d, connections missing comments: %d", numUpdates, numMissingComments)
 
 	if numUpdates+numMissingComments > 0 {
 		// get schema queries - this updates schemas for validated plugins and drops schemas for unvalidated plugins
 		s.executeUpdateQueries(ctx)
-	} else if len(connectionUpdates.Delete) > 0 {
+		// done
+		return
+	}
+
+	if len(s.connectionUpdates.Delete) > 0 {
 		log.Printf("[INFO] deleted all unnecessary schemas - sending notification")
 
 		// if there are no updates and there ARE deletes, notify
 		// (is there are updates, deletes will be notified by executeUpdateQueries)
-		if err := s.sendPostgreSchemaNotification(ctx); err != nil {
+		if err := s.pluginManager.SendPostgresSchemaNotification(ctx); err != nil {
 			// just log
 			log.Printf("[WARN] failed to send schema deletion Postgres notification: %s", err.Error())
 		}
-
 	}
-
-	return
 }
 
 // execute all update queries
 // NOTE: this only sets res.Error if there is a failure to set update the connection state table
 // - all other connection based failures are recorded in the connection state table
 func (s *refreshConnectionState) executeUpdateQueries(ctx context.Context) {
-	utils.LogTime("db.executeUpdateQueries start")
-	defer utils.LogTime("db.executeUpdateQueries end")
+	log.Println("[DEBUG] refreshConnectionState.executeUpdateQueries start")
+	defer log.Println("[DEBUG] refreshConnectionState.executeUpdateQueries end")
 
 	defer func() {
 		if s.res.Error != nil {
@@ -260,7 +287,7 @@ func (s *refreshConnectionState) executeUpdateQueries(ctx context.Context) {
 	moreErrors := s.executeUpdatesInParallel(ctx, initialUpdates)
 	errors = append(errors, moreErrors...)
 
-	// execute dynamic updates (not, we update all connections in search path order,
+	// execute dynamic updates (note, we update all connections in search path order,
 	// so must call executeUpdateSetsInParallel)
 	log.Printf("[INFO] executing dynamic updates")
 	moreErrors = s.executeUpdateSetsInParallel(ctx, dynamicUpdates)
@@ -271,7 +298,6 @@ func (s *refreshConnectionState) executeUpdateQueries(ctx context.Context) {
 	if len(errors) > 0 {
 		s.res.Error = error_helpers.CombineErrors(errors...)
 		log.Printf("[WARN] initial updates failed: %s", s.res.Error.Error())
-		// TODO SEND ERROR NOTIFICATION
 		return
 	}
 
@@ -288,8 +314,7 @@ func (s *refreshConnectionState) executeUpdateQueries(ctx context.Context) {
 	log.Printf("[INFO] updated all exemplar schemas - sending notification")
 	// now that we have updated all exemplar schemars, send postgres notification
 	// this gives any attached interactive clients a chance to update their inspect data and autocomplete
-
-	if err := s.sendPostgreSchemaNotification(ctx); err != nil {
+	if err := s.pluginManager.SendPostgresSchemaNotification(ctx); err != nil {
 		// just log
 		log.Printf("[WARN] failed to send schem update Postgres notification: %s", err.Error())
 	}
@@ -332,7 +357,7 @@ func (s *refreshConnectionState) executeUpdateQueries(ctx context.Context) {
 	return
 }
 
-// convert map upd update sets (used for dynamic schemas) to an array of the underlying connection states
+// convert map update sets (used for dynamic schemas) to an array of the underlying connection states
 func updateSetMapToArray(updateSetMap map[string][]*steampipeconfig.ConnectionState) []*steampipeconfig.ConnectionState {
 	var res []*steampipeconfig.ConnectionState
 	for _, updates := range updateSetMap {
@@ -344,21 +369,26 @@ func updateSetMapToArray(updateSetMap map[string][]*steampipeconfig.ConnectionSt
 // create/update connections
 
 func (s *refreshConnectionState) executeUpdatesInParallel(ctx context.Context, updates map[string]*steampipeconfig.ConnectionState) (errors []error) {
-	// just call executeUpdateSetsInParallel
+	log.Println("[DEBUG] refreshConnectionState.executeUpdatesInParallel start")
+	defer log.Println("[DEBUG] refreshConnectionState.executeUpdatesInParallel end")
 
 	// convert updates to update sets
 	updatesAsSets := make(map[string][]*steampipeconfig.ConnectionState, len(updates))
 	for k, v := range updates {
 		updatesAsSets[k] = []*steampipeconfig.ConnectionState{v}
 	}
+	// just call executeUpdateSetsInParallel
 	return s.executeUpdateSetsInParallel(ctx, updatesAsSets)
 }
 
-// execute sets of updates in parallel - this is required as for dynamic plugins, we must updated all connections in
+// execute sets of updates in parallel - this is required as for dynamic plugins, we must update all connections in
 // search path order
 // - for convenience we also use this function for static connections by mapping the input data
 // from map[string]*steampipeconfig.ConnectionState to map[string][]*steampipeconfig.ConnectionState
 func (s *refreshConnectionState) executeUpdateSetsInParallel(ctx context.Context, updates map[string][]*steampipeconfig.ConnectionState) (errors []error) {
+	log.Println("[DEBUG] refreshConnectionState.executeUpdateSetsInParallel start")
+	defer log.Println("[DEBUG] refreshConnectionState.executeUpdateSetsInParallel end")
+
 	var wg sync.WaitGroup
 	var errChan = make(chan *connectionError)
 
@@ -427,6 +457,8 @@ func (s *refreshConnectionState) executeUpdateSetsInParallel(ctx context.Context
 
 // syncronously execute the update queries for one or more connections
 func (s *refreshConnectionState) executeUpdateForConnections(ctx context.Context, errChan chan *connectionError, cloneSchemaEnabled bool, connectionStates ...*steampipeconfig.ConnectionState) {
+	log.Println("[DEBUG] refreshConnectionState.executeUpdateForConnections start")
+	defer log.Println("[DEBUG] refreshConnectionState.executeUpdateForConnections end")
 
 	for _, connectionState := range connectionStates {
 		connectionName := connectionState.ConnectionName
@@ -438,7 +470,7 @@ func (s *refreshConnectionState) executeUpdateForConnections(ctx context.Context
 		exemplarSchemaName, haveExemplarSchema := s.exemplarSchemaMap[connectionState.Plugin]
 		if haveExemplarSchema && cloneSchemaEnabled {
 			// we can clone!
-			sql = getCloneSchemaQuery(sql, exemplarSchemaName, connectionState)
+			sql = getCloneSchemaQuery(exemplarSchemaName, connectionState)
 		} else {
 			// just get sql to execute update query, and update the connection state table, in a transaction
 			sql = db_common.GetUpdateConnectionQuery(connectionName, remoteSchema)
@@ -459,7 +491,10 @@ func (s *refreshConnectionState) executeUpdateForConnections(ctx context.Context
 	}
 }
 
-func (s *refreshConnectionState) executeUpdateQuery(ctx context.Context, sql, connectionName string) error {
+func (s *refreshConnectionState) executeUpdateQuery(ctx context.Context, sql, connectionName string) (err error) {
+	log.Println("[DEBUG] refreshConnectionState.executeUpdateQuery start")
+	defer log.Println("[DEBUG] refreshConnectionState.executeUpdateQuery end")
+
 	// create a transaction
 	tx, err := s.pool.Begin(ctx)
 	if err != nil {
@@ -482,6 +517,7 @@ func (s *refreshConnectionState) executeUpdateQuery(ctx context.Context, sql, co
 		// update the state table
 		//(the transaction will be aborted - create a connection for the update)
 		if conn, poolErr := s.pool.Acquire(ctx); poolErr == nil {
+			defer conn.Release()
 			if statusErr := s.tableUpdater.onConnectionError(ctx, conn.Conn(), connectionName, err); statusErr != nil {
 				// NOTE: do not return the error - unless we failed to update the connection state table
 				return error_helpers.CombineErrorsWithPrefix(fmt.Sprintf("failed to update connection %s and failed to update connection_state table", connectionName), err, statusErr)
@@ -617,6 +653,7 @@ func (s *refreshConnectionState) executeCommentQuery(ctx context.Context, sql, c
 		// update the state table
 		//(the transaction will be aborted - create a connection for the update)
 		if conn, poolErr := s.pool.Acquire(ctx); poolErr == nil {
+			defer conn.Release()
 			if statusErr := s.tableUpdater.onConnectionError(ctx, conn.Conn(), connectionName, err); statusErr != nil {
 				// NOTE: do not return the error - unless we failed to update the connection state table
 				return error_helpers.CombineErrorsWithPrefix(fmt.Sprintf("failed to update connection %s and failed to update connection_state table", connectionName), err, statusErr)
@@ -635,14 +672,8 @@ func (s *refreshConnectionState) executeCommentQuery(ctx context.Context, sql, c
 	return nil
 }
 
-func getCloneSchemaQuery(sql string, exemplarSchemaName string, connectionState *steampipeconfig.ConnectionState) string {
-	sql = fmt.Sprintf("select clone_foreign_schema('%s', '%s', '%s');", exemplarSchemaName, connectionState.ConnectionName, connectionState.Plugin)
-	return sql
-}
-
-func getCloneCommentsQuery(sql string, exemplarSchemaName string, connectionState *steampipeconfig.ConnectionState) string {
-	sql = fmt.Sprintf("select clone_table_comments('%s', '%s');", exemplarSchemaName, connectionState.ConnectionName)
-	return sql
+func getCloneSchemaQuery(exemplarSchemaName string, connectionState *steampipeconfig.ConnectionState) string {
+	return fmt.Sprintf("select clone_foreign_schema('%s', '%s', '%s');", exemplarSchemaName, connectionState.ConnectionName, connectionState.Plugin)
 }
 
 func (s *refreshConnectionState) getInitialAndRemainingUpdates() (initialUpdates, remainingUpdates map[string]*steampipeconfig.ConnectionState, dynamicUpdates map[string][]*steampipeconfig.ConnectionState) {
@@ -659,7 +690,8 @@ func (s *refreshConnectionState) getInitialAndRemainingUpdates() (initialUpdates
 	for _, connectionName := range searchPathConnections {
 		if connectionState, updateRequired := updates[connectionName]; updateRequired {
 			if connectionState.SchemaMode == plugin.SchemaModeDynamic {
-				dynamicUpdates[connectionState.Plugin] = append(dynamicUpdates[connectionState.Plugin], connectionState)
+				pluginInstance := *connectionState.PluginInstance
+				dynamicUpdates[pluginInstance] = append(dynamicUpdates[pluginInstance], connectionState)
 			} else {
 				initialUpdates[connectionName] = connectionState
 			}
@@ -667,7 +699,8 @@ func (s *refreshConnectionState) getInitialAndRemainingUpdates() (initialUpdates
 	}
 	// now add remaining updates to remainingUpdates
 	for connectionName, connectionState := range updates {
-		if _, isInitialUpdate := initialUpdates[connectionName]; !isInitialUpdate {
+		_, isInitialUpdate := initialUpdates[connectionName]
+		if connectionState.SchemaMode == plugin.SchemaModeStatic && !isInitialUpdate {
 			remainingUpdates[connectionName] = connectionState
 		}
 
@@ -717,6 +750,7 @@ func (s *refreshConnectionState) executeDeleteQuery(ctx context.Context, connect
 		// update the state table
 		//(the transaction will be aborted - create a connection for the update)
 		if conn, poolErr := s.pool.Acquire(ctx); poolErr == nil {
+			defer conn.Release()
 			if statusErr := s.tableUpdater.onConnectionError(ctx, conn.Conn(), connectionName, err); statusErr != nil {
 				// NOTE: do not return the error - unless we failed to update the connection state table
 				return error_helpers.CombineErrorsWithPrefix(fmt.Sprintf("failed to update connection %s and failed to update connection_state table", connectionName), err, statusErr)
@@ -746,22 +780,10 @@ func (s *refreshConnectionState) setIncompleteConnectionStateToError(ctx context
 	}
 	defer conn.Release()
 
-	query := connection_state.GetIncompleteConnectionStateErrorSql(connectionStateError)
+	queries := introspection.GetIncompleteConnectionStateErrorSql(connectionStateError)
 
-	if _, err = db_local.ExecuteSqlWithArgsInTransaction(ctx, conn.Conn(), query); err != nil {
+	if _, err = db_local.ExecuteSqlWithArgsInTransaction(ctx, conn.Conn(), queries...); err != nil {
 		log.Printf("[WARN] setAllConnectionStateToError failed to set connection states to error: %s", err.Error())
 		return
 	}
 }
-
-// OnConnectionsChanged is the callback function invoked by the connection watcher when connections are added or removed
-func (s *refreshConnectionState) sendPostgreSchemaNotification(ctx context.Context) error {
-	conn, err := db_local.CreateLocalDbConnection(ctx, &db_local.CreateDbOptions{Username: constants.DatabaseSuperUser})
-	if err != nil {
-		return err
-	}
-	defer conn.Close(ctx)
-	notification := steampipeconfig.NewSchemaUpdateNotification(steampipeconfig.PgNotificationSchemaUpdate)
-
-	return db_local.SendPostgresNotification(ctx, conn, notification)
-}
