@@ -2,8 +2,7 @@ package connection_sync
 
 import (
 	"context"
-	"github.com/turbot/steampipe/pkg/constants"
-	"github.com/turbot/steampipe/pkg/db/db_client"
+
 	"github.com/turbot/steampipe/pkg/db/db_common"
 	"github.com/turbot/steampipe/pkg/steampipeconfig"
 )
@@ -13,7 +12,7 @@ import (
 // if any of the connections are in error state, return an error
 // this is used to ensure unqualified queries and tables are resolved to the correct connection
 func WaitForSearchPathSchemas(ctx context.Context, client db_common.Client, searchPath []string) error {
-	conn, err := client.AcquireConnection(ctx)
+	conn, err := client.AcquireManagementConnection(ctx)
 	if err != nil {
 		return err
 	}
@@ -23,8 +22,7 @@ func WaitForSearchPathSchemas(ctx context.Context, client db_common.Client, sear
 
 	// NOTE: if we failed to load conection state, this must be because we are connected to an older version of the CLI
 	// just return nil error
-	_, missingTable, relationNotFound := db_client.IsRelationNotFoundError(err)
-	if relationNotFound && missingTable == constants.ConnectionStateTable {
+	if db_common.IsRelationNotFoundError(err) {
 		return nil
 	}
 
