@@ -4,7 +4,6 @@ import (
 	"bufio"
 	"context"
 	"fmt"
-	"github.com/turbot/go-kit/files"
 	"io/fs"
 	"log"
 	"os"
@@ -14,6 +13,8 @@ import (
 	"strings"
 	"time"
 
+	"github.com/turbot/go-kit/files"
+
 	"github.com/shirou/gopsutil/process"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/error_helpers"
@@ -84,6 +85,7 @@ func prepareBackup(ctx context.Context) (*string, error) {
 		log.Printf("[TRACE] Error while starting old db in %s: %v", location, err)
 		return nil, err
 	}
+	//nolint:golint,errcheck // this will probably never error - if it does, it's not something we can recover from with code
 	defer runConfig.stop(ctx)
 
 	if err := takeBackup(ctx, runConfig); err != nil {
@@ -125,14 +127,14 @@ func killRunningDbInstance(ctx context.Context) error {
 func takeBackup(ctx context.Context, config *pgRunningInfo) error {
 	cmd := pgDumpCmd(
 		ctx,
-		fmt.Sprintf("--file=%s", databaseBackupFilePath()),
+		fmt.Sprintf("--file=%s", filepaths.DatabaseBackupFilePath()),
 		fmt.Sprintf("--format=%s", backupFormat),
 		// of the public schema only
 		"--schema=public",
 		// only backup the database used by steampipe
 		fmt.Sprintf("--dbname=%s", config.dbName),
 		// connection parameters
-		"--host=localhost",
+		"--host=127.0.0.1",
 		fmt.Sprintf("--port=%d", config.port),
 		fmt.Sprintf("--username=%s", constants.DatabaseSuperUser),
 	)
@@ -158,9 +160,9 @@ func startDatabaseInLocation(ctx context.Context, location string) (*pgRunningIn
 	cmd := exec.CommandContext(
 		ctx,
 		binaryLocation,
-		// by this time, we are sure that the port if free to listen to
+		// by this time, we are sure that the port is free to listen to
 		"-p", fmt.Sprint(port),
-		"-c", "listen_addresses=localhost",
+		"-c", "listen_addresses=127.0.0.1",
 		// NOTE: If quoted, the application name includes the quotes. Worried about
 		// having spaces in the APPNAME, but leaving it unquoted since currently
 		// the APPNAME is hardcoded to be steampipe.
@@ -227,7 +229,7 @@ func findDifferentPgInstallation(ctx context.Context) (bool, string, error) {
 
 // restoreDBBackup loads the back up file into the database
 func restoreDBBackup(ctx context.Context) error {
-	backupFilePath := databaseBackupFilePath()
+	backupFilePath := filepaths.DatabaseBackupFilePath()
 	if !files.FileExists(backupFilePath) {
 		// nothing to do here
 		return nil
@@ -312,7 +314,7 @@ func restoreDBBackup(ctx context.Context) error {
 func runRestoreUsingList(ctx context.Context, info *RunningDBInstanceInfo, listFile string) error {
 	cmd := pgRestoreCmd(
 		ctx,
-		databaseBackupFilePath(),
+		filepaths.DatabaseBackupFilePath(),
 		fmt.Sprintf("--format=%s", backupFormat),
 		// only the public schema is backed up
 		"--schema=public",
@@ -325,7 +327,7 @@ func runRestoreUsingList(ctx context.Context, info *RunningDBInstanceInfo, listF
 		// the database name
 		fmt.Sprintf("--dbname=%s", info.Database),
 		// connection parameters
-		"--host=localhost",
+		"--host=127.0.0.1",
 		fmt.Sprintf("--port=%d", info.Port),
 		fmt.Sprintf("--username=%s", info.User),
 	)
@@ -367,7 +369,7 @@ func partitionTableOfContents(ctx context.Context, tableOfContentsOfBackup []str
 func getTableOfContentsFromBackup(ctx context.Context) ([]string, error) {
 	cmd := pgRestoreCmd(
 		ctx,
-		databaseBackupFilePath(),
+		filepaths.DatabaseBackupFilePath(),
 		fmt.Sprintf("--format=%s", backupFormat),
 		// only the public schema is backed up
 		"--schema=public",
@@ -418,7 +420,7 @@ func retainBackup(ctx context.Context) error {
 	textBackupFilePath := filepath.Join(backupDir, textBackupRetentionFileName)
 
 	log.Println("[TRACE] moving database back up to", binaryBackupFilePath)
-	if err := utils.MoveFile(databaseBackupFilePath(), binaryBackupFilePath); err != nil {
+	if err := utils.MoveFile(filepaths.DatabaseBackupFilePath(), binaryBackupFilePath); err != nil {
 		return err
 	}
 	log.Println("[TRACE] converting database back up to", textBackupFilePath)
@@ -442,7 +444,7 @@ func retainBackup(ctx context.Context) error {
 func pgDumpCmd(ctx context.Context, args ...string) *exec.Cmd {
 	cmd := exec.CommandContext(
 		ctx,
-		pgDumpBinaryExecutablePath(),
+		filepaths.PgDumpBinaryExecutablePath(),
 		args...,
 	)
 	cmd.Env = append(os.Environ(), "PGSSLMODE=disable")
@@ -454,7 +456,7 @@ func pgDumpCmd(ctx context.Context, args ...string) *exec.Cmd {
 func pgRestoreCmd(ctx context.Context, args ...string) *exec.Cmd {
 	cmd := exec.CommandContext(
 		ctx,
-		pgRestoreBinaryExecutablePath(),
+		filepaths.PgRestoreBinaryExecutablePath(),
 		args...,
 	)
 	cmd.Env = append(os.Environ(), "PGSSLMODE=disable")
