@@ -28,14 +28,14 @@ var ensureMux sync.Mutex
 func noBackupWarning() string {
 	warningMessage := `Steampipe database has been upgraded from Postgres 12 to Postgres 14.
 
-Unfortunately the data in your public schema failed migration using the standard pg_dump and pg_restore tools. Your data has been preserved in the ~/.steampipe/db directory. 
+Unfortunately the data in your public schema failed migration using the standard pg_dump and pg_restore tools. Your data has been preserved in the ~/.steampipe/db directory.
 
 If you need to restore the contents of your public schema, please open an issue at https://github.com/turbot/steampipe.`
 
 	return fmt.Sprintf("%s: %v\n", color.YellowString("Warning"), warningMessage)
 }
 
-// EnsureDBInstalled makes sure that the embedded pg database is installed and running
+// EnsureDBInstalled makes sure that the embedded postgres database is installed and ready to run
 func EnsureDBInstalled(ctx context.Context) (err error) {
 	utils.LogTime("db_local.EnsureDBInstalled start")
 
@@ -52,8 +52,8 @@ func EnsureDBInstalled(ctx context.Context) (err error) {
 		close(doneChan)
 	}()
 
-	if IsInstalled() {
-		// check if the FDW need updating, and init the db id required
+	if IsDBInstalled() {
+		// check if the FDW need updating, and init the db if required
 		err := prepareDb(ctx)
 		return err
 	}
@@ -65,7 +65,7 @@ func EnsureDBInstalled(ctx context.Context) (err error) {
 		return err
 	}
 	if dbState != nil {
-		return fmt.Errorf("cannot install db - a previous version of the Steampipe service is still running. To stop running services, use %s ", constants.Bold("steampipe service stop"))
+		return fmt.Errorf("cannot install service - a previous version of the Steampipe service is still running. To stop running services, use %s ", constants.Bold("steampipe service stop"))
 	}
 
 	log.Println("[TRACE] calling removeRunningInstanceInfo")
@@ -90,7 +90,7 @@ func EnsureDBInstalled(ctx context.Context) (err error) {
 	if err != nil {
 		if errors.Is(err, errDbInstanceRunning) {
 			// remove the installation - otherwise, the backup won't get triggered, even if the user stops the service
-			os.RemoveAll(databaseInstanceDir())
+			os.RemoveAll(filepaths.DatabaseInstanceDir())
 			return err
 		}
 		// ignore all other errors with the backup, displaying a warning instead
@@ -125,14 +125,14 @@ func EnsureDBInstalled(ctx context.Context) (err error) {
 func downloadAndInstallDbFiles(ctx context.Context) error {
 	statushooks.SetStatus(ctx, "Prepare database install location…")
 	// clear all db files
-	err := os.RemoveAll(getDatabaseLocation())
+	err := os.RemoveAll(filepaths.GetDatabaseLocation())
 	if err != nil {
 		log.Printf("[TRACE] %v", err)
 		return fmt.Errorf("Prepare database install location... FAILED!")
 	}
 
 	statushooks.SetStatus(ctx, "Download & install embedded PostgreSQL database…")
-	_, err = ociinstaller.InstallDB(ctx, getDatabaseLocation())
+	_, err = ociinstaller.InstallDB(ctx, filepaths.GetDatabaseLocation())
 	if err != nil {
 		log.Printf("[TRACE] %v", err)
 		return fmt.Errorf("Download & install embedded PostgreSQL database... FAILED!")
@@ -140,35 +140,32 @@ func downloadAndInstallDbFiles(ctx context.Context) error {
 	return nil
 }
 
-// IsInstalled checks and reports whether the embedded database is installed and setup
-func IsInstalled() bool {
+// IsDBInstalled checks and reports whether the embedded database binaries are available
+func IsDBInstalled() bool {
 	utils.LogTime("db_local.IsInstalled start")
 	defer utils.LogTime("db_local.IsInstalled end")
-
 	// check that both postgres binary and initdb binary exist
-	// and are executable by us
-
-	if _, err := os.Stat(getInitDbBinaryExecutablePath()); os.IsNotExist(err) {
+	if _, err := os.Stat(filepaths.GetInitDbBinaryExecutablePath()); os.IsNotExist(err) {
 		return false
 	}
-
-	if _, err := os.Stat(getPostgresBinaryExecutablePath()); os.IsNotExist(err) {
+	if _, err := os.Stat(filepaths.GetPostgresBinaryExecutablePath()); os.IsNotExist(err) {
 		return false
 	}
+	return true
+}
 
-	if _, err := os.Stat(getFDWBinaryLocation()); os.IsNotExist(err) {
-		return false
-	}
-
-	fdwSQLFile, fdwControlFile := getFDWSQLAndControlLocation()
-
+// IsFDWInstalled chceks whether all files required for the Steampipe FDW are available
+func IsFDWInstalled() bool {
+	fdwSQLFile, fdwControlFile := filepaths.GetFDWSQLAndControlLocation()
 	if _, err := os.Stat(fdwSQLFile); os.IsNotExist(err) {
 		return false
 	}
 	if _, err := os.Stat(fdwControlFile); os.IsNotExist(err) {
 		return false
 	}
-
+	if _, err := os.Stat(filepaths.GetFDWBinaryLocation()); os.IsNotExist(err) {
+		return false
+	}
 	return true
 }
 
@@ -199,17 +196,15 @@ func prepareDb(ctx context.Context) error {
 			log.Printf("[TRACE] updateDownloadedBinarySignature failed: %v", err)
 			return fmt.Errorf("Updating install records... FAILED!")
 		}
+	}
 
+	// if the FDW is not installed, or needs an update
+	if !IsFDWInstalled() || fdwNeedsUpdate(versionInfo) {
 		// install fdw
 		if _, err := installFDW(ctx, false); err != nil {
 			log.Printf("[TRACE] installFDW failed: %v", err)
 			return fmt.Errorf("Update steampipe-postgres-fdw... FAILED!")
 		}
-	} else if fdwNeedsUpdate(versionInfo) {
-		if _, err := installFDW(ctx, false); err != nil {
-			log.Printf("[TRACE] installFDW failed: %v", err)
-			return fmt.Errorf("Update steampipe-postgres-fdw... FAILED!")
-		}
 
 		// get the message renderer from the context
 		// this allows the interactive client init to inject a custom renderer
@@ -252,7 +247,7 @@ func installFDW(ctx context.Context, firstSetup bool) (string, error) {
 		}()
 	}
 	statushooks.SetStatus(ctx, fmt.Sprintf("Download & install %s…", constants.Bold("steampipe-postgres-fdw")))
-	return ociinstaller.InstallFdw(ctx, getDatabaseLocation())
+	return ociinstaller.InstallFdw(ctx, filepaths.GetDatabaseLocation())
 }
 
 func needsInit() bool {
@@ -260,7 +255,7 @@ func needsInit() bool {
 	defer utils.LogTime("db_local.needsInit end")
 
 	// test whether pg_hba.conf exists in our target directory
-	return !filehelpers.FileExists(getPgHbaConfLocation())
+	return !filehelpers.FileExists(filepaths.GetPgHbaConfLocation())
 }
 
 func runInstall(ctx context.Context, oldDbName *string) error {
@@ -269,7 +264,7 @@ func runInstall(ctx context.Context, oldDbName *string) error {
 
 	statushooks.SetStatus(ctx, "Cleaning up…")
 
-	err := utils.RemoveDirectoryContents(getDataLocation())
+	err := utils.RemoveDirectoryContents(filepaths.GetDataLocation())
 	if err != nil {
 		log.Printf("[TRACE] %v", err)
 		return fmt.Errorf("Prepare database install location... FAILED!")
@@ -317,15 +312,7 @@ func runInstall(ctx context.Context, oldDbName *string) error {
 	// resolve the name of the database that is to be installed
 	databaseName := resolveDatabaseName(oldDbName)
 	// validate db name
-	firstCharacter := databaseName[0:1]
-	var ascii int
-	for _, r := range databaseName {
-		ascii = int(r)
-		break
-	}
-	if firstCharacter == "_" || (ascii >= 'a' && ascii <= 'z') {
-		log.Printf("[TRACE] valid database name: %s", databaseName)
-	} else {
+	if !isValidDatabaseName(databaseName) {
 		return fmt.Errorf("Invalid database name '%s' - must start with either a lowercase character or an underscore", databaseName)
 	}
 
@@ -362,10 +349,10 @@ func resolveDatabaseName(oldDbName *string) string {
 
 func startServiceForInstall(port int) (*psutils.Process, error) {
 	postgresCmd := exec.Command(
-		getPostgresBinaryExecutablePath(),
+		filepaths.GetPostgresBinaryExecutablePath(),
 		// by this time, we are sure that the port if free to listen to
 		"-p", fmt.Sprint(port),
-		"-c", "listen_addresses=localhost",
+		"-c", "listen_addresses=127.0.0.1",
 		// NOTE: If quoted, the application name includes the quotes. Worried about
 		// having spaces in the APPNAME, but leaving it unquoted since currently
 		// the APPNAME is hardcoded to be steampipe.
@@ -376,7 +363,7 @@ func startServiceForInstall(port int) (*psutils.Process, error) {
 		"-c", fmt.Sprintf("log_directory=%s", filepaths.EnsureLogDir()),
 
 		// Data Directory
-		"-D", getDataLocation())
+		"-D", filepaths.GetDataLocation())
 
 	setupLogCollection(postgresCmd)
 
@@ -388,6 +375,10 @@ func startServiceForInstall(port int) (*psutils.Process, error) {
 	return psutils.NewProcess(int32(postgresCmd.Process.Pid))
 }
 
+func isValidDatabaseName(databaseName string) bool {
+	return databaseName[0] == '_' || (databaseName[0] >= 'a' && databaseName[0] <= 'z')
+}
+
 func getNextFreePort() (int, error) {
 	utils.LogTime("db_local.install.getNextFreePort start")
 	defer utils.LogTime("db_local.install.getNextFreePort end")
@@ -417,7 +408,7 @@ func initDatabase() error {
 		return err
 	}
 
-	initDBExecutable := getInitDbBinaryExecutablePath()
+	initDBExecutable := filepaths.GetInitDbBinaryExecutablePath()
 	initDbProcess := exec.Command(
 		initDBExecutable,
 		// Steampipe runs Postgres as a local, embedded database so trust local
@@ -428,7 +419,7 @@ func initDatabase() error {
 		// embedded database.
 		fmt.Sprintf("--username=%s", constants.DatabaseSuperUser),
 		// Postgres data should placed under the Steampipe install directory.
-		fmt.Sprintf("--pgdata=%s", getDataLocation()),
+		fmt.Sprintf("--pgdata=%s", filepaths.GetDataLocation()),
 		// Ensure the encoding is consistent across installs. By default it would
 		// be based on the system locale.
 		fmt.Sprintf("--encoding=%s", "UTF-8"),
@@ -451,7 +442,7 @@ func initDatabase() error {
 
 	// intentionally overwriting existing pg_hba.conf with a minimal config which only allows root
 	// so that we can setup the database and permissions
-	return os.WriteFile(getPgHbaConfLocation(), []byte(constants.MinimalPgHbaContent), 0600)
+	return os.WriteFile(filepaths.GetPgHbaConfLocation(), []byte(constants.MinimalPgHbaContent), 0600)
 }
 
 func installDatabaseWithPermissions(ctx context.Context, databaseName string, rawClient *pgx.Conn) error {
@@ -524,7 +515,7 @@ func installDatabaseWithPermissions(ctx context.Context, databaseName string, ra
 
 func writePgHbaContent(databaseName string, username string) error {
 	content := fmt.Sprintf(constants.PgHbaTemplate, databaseName, username)
-	return os.WriteFile(getPgHbaConfLocation(), []byte(content), 0600)
+	return os.WriteFile(filepaths.GetPgHbaConfLocation(), []byte(content), 0600)
 }
 
 func installForeignServer(ctx context.Context, rawClient *pgx.Conn) error {
@@ -560,5 +551,5 @@ func updateDownloadedBinarySignature() error {
 		return err
 	}
 	installedSignature := fmt.Sprintf("%s|%s", versionInfo.EmbeddedDB.ImageDigest, versionInfo.FdwExtension.ImageDigest)
-	return os.WriteFile(getDBSignatureLocation(), []byte(installedSignature), 0755)
+	return os.WriteFile(filepaths.GetDBSignatureLocation(), []byte(installedSignature), 0755)
 }
