@@ -16,7 +16,6 @@ import (
 	"github.com/alecthomas/chroma/lexers"
 	"github.com/alecthomas/chroma/styles"
 	"github.com/c-bata/go-prompt"
-	"github.com/jackc/pgx/v5"
 	"github.com/jackc/pgx/v5/pgconn"
 	"github.com/spf13/viper"
 	"github.com/turbot/go-kit/helpers"
@@ -24,7 +23,6 @@ import (
 	"github.com/turbot/steampipe/pkg/connection_sync"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/db/db_common"
-	"github.com/turbot/steampipe/pkg/db/db_local"
 	"github.com/turbot/steampipe/pkg/display"
 	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/interactive/metaquery"
@@ -56,11 +54,6 @@ type InteractiveClient struct {
 	// NOTE: should ONLY be called by cancelActiveQueryIfAny
 	cancelActiveQuery context.CancelFunc
 	cancelPrompt      context.CancelFunc
-	// this cancellation is used to stop the pg notification listener which
-	// we use to get connection config updates from the plugin manager
-	// this is tied to a context which remaing valid throughout the life of the
-	// interactive session
-	cancelNotificationListener context.CancelFunc
 
 	// channel used internally to pass the initialisation result
 	initResultChan chan *db_common.InitResult
@@ -142,8 +135,8 @@ func (c *InteractiveClient) InteractivePrompt(parentContext context.Context) {
 	)
 
 	// run the prompt in a goroutine, so we can also detect async initialisation errors
-	promptResultChan := make(chan utils.InteractiveExitStatus, 1)
-	c.runInteractivePromptAsync(ctx, &promptResultChan)
+	promptResultChan := make(chan struct{}, 1)
+	c.runInteractivePromptAsync(ctx, promptResultChan)
 
 	// select results
 	for {
@@ -155,23 +148,19 @@ func (c *InteractiveClient) InteractivePrompt(parentContext context.Context) {
 
 		case <-promptResultChan:
 			// persist saved history
+			//nolint:golint,errcheck // worst case is history is not persisted - not a failure
 			c.interactiveQueryHistory.Persist()
 			// check post-close action
 			if c.afterClose == AfterPromptCloseExit {
 				// clear prompt so any messages/warnings can be displayed without the prompt
 				c.hidePrompt = true
 				c.interactivePrompt.ClearLine()
-
-				// stop the notification listener
-				if c.cancelNotificationListener != nil {
-					c.cancelNotificationListener()
-				}
 				return
 			}
 			// create new context with a cancellation func
 			ctx = c.createPromptContext(parentContext)
 			// now run it again
-			c.runInteractivePromptAsync(ctx, &promptResultChan)
+			c.runInteractivePromptAsync(ctx, promptResultChan)
 		}
 	}
 }
@@ -198,13 +187,14 @@ func (c *InteractiveClient) loadSchema() error {
 	return nil
 }
 
-func (c *InteractiveClient) runInteractivePromptAsync(ctx context.Context, promptResultChan *chan utils.InteractiveExitStatus) {
+func (c *InteractiveClient) runInteractivePromptAsync(ctx context.Context, promptResultChan chan struct{}) {
 	go func() {
-		*promptResultChan <- c.runInteractivePrompt(ctx)
+		c.runInteractivePrompt(ctx)
+		promptResultChan <- struct{}{}
 	}()
 }
 
-func (c *InteractiveClient) runInteractivePrompt(ctx context.Context) (ret utils.InteractiveExitStatus) {
+func (c *InteractiveClient) runInteractivePrompt(ctx context.Context) {
 	defer func() {
 		// this is to catch the PANIC that gets raised by
 		// the executor of go-prompt
@@ -213,17 +203,12 @@ func (c *InteractiveClient) runInteractivePrompt(ctx context.Context) (ret utils
 		// clean way to reload go-prompt so that we can
 		// populate the history stack
 		//
-		r := recover()
-		switch v := r.(type) {
-		case utils.InteractiveExitStatus:
-			// this is a planned exit
-			// set the return value
-			ret = v
-		default:
-			if r != nil {
-				// for everything else, float up the panic
-				panic(r)
-			}
+		if r := recover(); r != nil {
+			// show the panic and restart the prompt
+			error_helpers.ShowError(ctx, helpers.ToError(r))
+			c.afterClose = AfterPromptCloseRestart
+			c.hidePrompt = false
+			return
 		}
 	}()
 
@@ -566,7 +551,7 @@ func (c *InteractiveClient) getConnectionState(ctx context.Context) (steampipeco
 
 	statushooks.SetStatus(ctx, "Loading connection state…")
 
-	conn, err := c.client().AcquireConnection(ctx)
+	conn, err := c.client().AcquireManagementConnection(ctx)
 	if err != nil {
 		return nil, err
 	}
@@ -648,6 +633,7 @@ func (c *InteractiveClient) getFirstWordSuggestions(word string) []prompt.Sugges
 			querySuggestions = modQueries
 		} else {
 			//  otherwise return mods names and unqualified queries
+			//nolint:golint,gocritic // we want this to go into a different slice
 			querySuggestions = append(c.suggestions.mods, c.suggestions.unqualifiedQueries...)
 		}
 	}
@@ -691,7 +677,7 @@ func (c *InteractiveClient) startCancelHandler() chan bool {
 		for {
 			select {
 			case <-sigIntChannel:
-				log.Println("[TRACE] interactive client cancel handler got SIGINT")
+				log.Println("[INFO] interactive client cancel handler got SIGINT")
 				// if initialisation is not complete, just close the prompt
 				// this will cancel the context used for initialisation so cancel any initialisation queries
 				if !c.isInitialised() {
@@ -703,7 +689,7 @@ func (c *InteractiveClient) startCancelHandler() chan bool {
 					// keep waiting for further cancellations
 				}
 			case <-quitChannel:
-				log.Println("[TRACE] cancel handler exiting")
+				log.Println("[INFO] cancel handler exiting")
 				c.cancelActiveQueryIfAny()
 				// we're done
 				return
@@ -713,57 +699,16 @@ func (c *InteractiveClient) startCancelHandler() chan bool {
 	return quitChannel
 }
 
-func (c *InteractiveClient) listenToPgNotifications(ctx context.Context) error {
-	log.Printf("[TRACE] InteractiveClient listenToPgNotifications")
-	conn, err := c.getNotificationConnection(ctx)
-	if err != nil {
-		return err
-	}
-	for ctx.Err() == nil {
-		if err != nil {
-			return err
-		}
-
-		log.Printf("[TRACE] Wait for notification")
-		notification, err := conn.WaitForNotification(ctx)
-		if err != nil && !error_helpers.IsContextCancelledError(err) {
-			log.Printf("[INFO] Error waiting for notification: %s", err)
-			// TODO what to do about connection closed error
-			return err
-		}
-
-		if notification != nil {
-			c.handlePostgresNotification(ctx, notification)
-		}
-		log.Printf("[TRACE] Handled notification")
-	}
-	conn.Close(ctx)
-
-	log.Printf("[TRACE] InteractiveClient listenToPgNotifications DONE")
-	return nil
-}
-
-func (c *InteractiveClient) getNotificationConnection(ctx context.Context) (*pgx.Conn, error) {
-	conn, err := db_local.CreateLocalDbConnection(ctx, &db_local.CreateDbOptions{Username: constants.DatabaseUser})
-	if err != nil {
-		return nil, err
-	}
-
-	listenSql := fmt.Sprintf("listen %s", constants.PostgresNotificationChannel)
-	_, err = conn.Exec(ctx, listenSql)
-	if err != nil {
-		log.Printf("[INFO] Error listening to schema channel: %s", err)
-		conn.Close(ctx)
-		return nil, err
-	}
-	return conn, nil
+func (c *InteractiveClient) listenToPgNotifications(ctx context.Context) {
+	c.initData.Client.RegisterNotificationListener(func(notification *pgconn.Notification) {
+		c.handlePostgresNotification(ctx, notification)
+	})
 }
 
 func (c *InteractiveClient) handlePostgresNotification(ctx context.Context, notification *pgconn.Notification) {
 	if notification == nil {
 		return
 	}
-	log.Printf("[TRACE] handleConnectionUpdateNotification")
 	n := &steampipeconfig.PostgresNotification{}
 	err := json.Unmarshal([]byte(notification.Payload), n)
 	if err != nil {
@@ -772,41 +717,69 @@ func (c *InteractiveClient) handlePostgresNotification(ctx context.Context, noti
 	}
 	switch n.Type {
 	case steampipeconfig.PgNotificationSchemaUpdate:
+		c.handleConnectionUpdateNotification(ctx)
+	case steampipeconfig.PgNotificationConnectionError:
 		// unmarshal the notification again, into the correct type
-		schemaUpdateNotification := &steampipeconfig.SteampipeNotification{}
-		if err := json.Unmarshal([]byte(notification.Payload), schemaUpdateNotification); err != nil {
+		errorNotification := &steampipeconfig.ErrorsAndWarningsNotification{}
+		if err := json.Unmarshal([]byte(notification.Payload), errorNotification); err != nil {
 			log.Printf("[WARN] Error unmarshalling notification: %s", err)
 			return
 		}
-		c.handleConnectionUpdateNotification(ctx)
+		c.handleErrorsAndWarningsNotification(ctx, errorNotification)
 	}
 }
 
+func (c *InteractiveClient) handleErrorsAndWarningsNotification(ctx context.Context, notification *steampipeconfig.ErrorsAndWarningsNotification) {
+	log.Printf("[TRACE] handleErrorsAndWarningsNotification")
+	output := viper.Get(constants.ArgOutput)
+	if output == constants.OutputFormatJSON || output == constants.OutputFormatCSV {
+		return
+	}
+
+	c.showMessages(ctx, func() {
+		for _, m := range append(notification.Errors, notification.Warnings...) {
+			error_helpers.ShowWarning(m)
+		}
+	})
+
+}
 func (c *InteractiveClient) handleConnectionUpdateNotification(ctx context.Context) {
+	// ignore schema update notifications until initialisation is complete
+	// (we may receive schema update messages from the initial refresh connections, but we do not need to reload
+	// the schema as we will have already loaded the correct schema)
+	if !c.initialisationComplete {
+		log.Printf("[INFO] received schema update notification but ignoring it as we are initializing")
+		return
+	}
+
 	// at present, we do not actually use the payload, we just do a brute force reload
 	// as an optimization we could look at the updates and only reload the required schemas
 
-	log.Printf("[TRACE] handleConnectionUpdateNotification")
+	log.Printf("[INFO] handleConnectionUpdateNotification")
 
 	// first load user search path
 	if err := c.client().LoadUserSearchPath(ctx); err != nil {
-		log.Printf("[INFO] Error in handleConnectionUpdateNotification when loading foreign user search path: %s", err.Error())
+		log.Printf("[WARN] Error in handleConnectionUpdateNotification when loading foreign user search path: %s", err.Error())
 		return
 	}
 
 	//  reload schema
 	if err := c.loadSchema(); err != nil {
-		log.Printf("[INFO] Error unmarshalling notification: %s", err)
+		log.Printf("[WARN] Error unmarshalling notification: %s", err)
 		return
 	}
 
 	// reinitialise autocomplete suggestions
-	c.initialiseSuggestions(ctx)
+
+	if err := c.initialiseSuggestions(ctx); err != nil {
+		log.Printf("[WARN] failed to initialise suggestions: %s", err)
+	}
 
 	// refresh the db session inside an execution lock
 	// we do this to avoid the postgres `cached plan must not change result type`` error
 	c.executionLock.Lock()
 	defer c.executionLock.Unlock()
 
-	c.client().RefreshSessions(ctx)
+	// refresh all connections in the pool - since the search path may have changed
+	c.client().ResetPools(ctx)
 }
