@@ -2,7 +2,9 @@ package steampipeconfig
 
 import (
 	"fmt"
+	typehelpers "github.com/turbot/go-kit/types"
 	"log"
+	"strings"
 
 	"github.com/hashicorp/go-plugin"
 	sdkgrpc "github.com/turbot/steampipe-plugin-sdk/v5/grpc"
@@ -10,11 +12,12 @@ import (
 	sdkplugin "github.com/turbot/steampipe-plugin-sdk/v5/plugin"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/error_helpers"
-	"github.com/turbot/steampipe/pkg/pluginmanager"
 	"github.com/turbot/steampipe/pkg/pluginmanager_service/grpc/proto"
+	pluginshared "github.com/turbot/steampipe/pkg/pluginmanager_service/grpc/shared"
 	"github.com/turbot/steampipe/pkg/steampipeconfig/modconfig"
 	"github.com/turbot/steampipe/pkg/steampipeconfig/options"
 	"github.com/turbot/steampipe/pkg/utils"
+	"golang.org/x/exp/maps"
 )
 
 type ConnectionPluginData struct {
@@ -35,6 +38,7 @@ type ConnectionPlugin struct {
 	PluginName          string
 	PluginClient        *sdkgrpc.PluginClient
 	SupportedOperations *proto.SupportedOperations
+	PluginShortName     string
 }
 
 func (p ConnectionPlugin) addConnection(name string, config string, connectionOptions *options.Connection, connectionType string) {
@@ -47,17 +51,24 @@ func (p ConnectionPlugin) addConnection(name string, config string, connectionOp
 }
 
 // GetSchema returns the cached schema if it is static, or if it is dynamic, refetch it
-func (p ConnectionPlugin) GetSchema(connectionName string) (*sdkproto.Schema, error) {
+func (p ConnectionPlugin) GetSchema(connectionName string) (schema *sdkproto.Schema, err error) {
+	defer func() {
+		if err != nil {
+			log.Printf("[TRACE] GetSchema for connection '%s' returning tables: %s", connectionName, strings.Join(maps.Keys(schema.Schema), ","))
+		}
+	}()
+	log.Printf("[TRACE] GetSchema for connection '%s'", connectionName)
 	connectionData, ok := p.ConnectionMap[connectionName]
 	if ok {
 		// if the schema mode is static, return the cached schema
 		if connectionData.Schema.Mode == sdkplugin.SchemaModeStatic {
+			log.Printf("[TRACE] connection data for connection '%s' is already loaded and schema is static - returning cached schema", connectionName)
 			return connectionData.Schema, nil
 		}
 	}
 	// otherwise this is a dynamic schema - refetch it
 	// we need to do this in case it has changed (for example as a result of a file watching event)
-	schema, err := p.PluginClient.GetSchema(connectionName)
+	schema, err = p.PluginClient.GetSchema(connectionName)
 	if err != nil {
 		log.Printf("[TRACE] failed to get schema for connection '%s': %s", connectionName, err)
 		return nil, err
@@ -68,8 +79,9 @@ func (p ConnectionPlugin) GetSchema(connectionName string) (*sdkproto.Schema, er
 	return schema, nil
 }
 
-func NewConnectionPlugin(pluginName string, pluginClient *sdkgrpc.PluginClient, supportedOperations *proto.SupportedOperations) *ConnectionPlugin {
+func NewConnectionPlugin(pluginShortName, pluginName string, pluginClient *sdkgrpc.PluginClient, supportedOperations *proto.SupportedOperations) *ConnectionPlugin {
 	return &ConnectionPlugin{
+		PluginShortName:     pluginShortName,
 		PluginName:          pluginName,
 		PluginClient:        pluginClient,
 		SupportedOperations: supportedOperations,
@@ -77,7 +89,10 @@ func NewConnectionPlugin(pluginName string, pluginClient *sdkgrpc.PluginClient,
 }
 
 // CreateConnectionPlugins instantiates plugins for specified connections, and fetches schemas
-func CreateConnectionPlugins(connectionNamesToCreate []string) (requestedConnectionPluginMap map[string]*ConnectionPlugin, res *RefreshConnectionResult) {
+func CreateConnectionPlugins(pluginManager pluginshared.PluginManager, connectionNamesToCreate []string) (requestedConnectionPluginMap map[string]*ConnectionPlugin, res *RefreshConnectionResult) {
+	log.Println("[DEBUG] CreateConnectionPlugins start")
+	defer log.Println("[DEBUG] CreateConnectionPlugins end")
+
 	res = &RefreshConnectionResult{}
 	requestedConnectionPluginMap = make(map[string]*ConnectionPlugin)
 	if len(connectionNamesToCreate) == 0 {
@@ -97,41 +112,32 @@ func CreateConnectionPlugins(connectionNamesToCreate []string) (requestedConnect
 		connectionNames[i] = connection.Name
 	}
 
-	// get plugin manager
-	pluginManager, err := pluginmanager.GetPluginManager()
-	if err != nil {
-		res.Error = err
-		return nil, res
-	}
-
 	// ask the plugin manager for the reattach config for all required plugins
 	getResponse, err := pluginManager.Get(&proto.GetRequest{Connections: connectionNames})
 	if err != nil {
 		res.Error = err
 		return nil, res
 	}
-
-	// if there were any failures, display them
-	for failedPlugin, failure := range getResponse.FailureMap {
-		res.AddWarning(fmt.Sprintf("failed to start plugin '%s': %s", failedPlugin, failure))
-		// figure out which connections are provided by any failed plugins
-		for _, c := range connectionsToCreate {
-			if c.Plugin == failedPlugin {
-				res.AddFailedConnection(c.Name, constants.ConnectionErrorPluginFailedToStart)
-			}
-		}
-	}
+	// construct friendly warning messages for any get failures
+	handleGetFailures(getResponse, res, connectionsToCreate)
 
 	// now create or retrieve a connection plugin for each connection
 
-	// NOTE: multiple connections may use the same plugin
-	// store a map of multi connection plugins, keyed by connection name
-	multiConnectionPlugins := make(map[string]*ConnectionPlugin)
+	// NOTE: multiple connections use the same plugin
+	// store a map of multi ConnectionPlugins, keyed by plugin name
+	connectionPluginMap := make(map[string]*ConnectionPlugin)
 
 	for _, connection := range connectionsToCreate {
+		// we must have a plugin instance
+		if connection.PluginInstance == nil {
+			// unexpected
+			res.AddWarning(fmt.Sprintf("connection '%s' has no plugin instance", connection.Name))
+			continue
+		}
+		pluginInstance := *connection.PluginInstance
 		// is this connection provided by a plugin we have already instantiated?
-		if existingConnectionPlugin, ok := multiConnectionPlugins[connection.Plugin]; ok {
-			log.Printf("[TRACE] CreateConnectionPlugins - connection %s is provided by existing connectionPlugin %s - reusing", connection.Name, connection.Plugin)
+		if existingConnectionPlugin, ok := connectionPluginMap[pluginInstance]; ok {
+			log.Printf("[TRACE] CreateConnectionPlugins - connection %s is provided by existing connectionPlugin %s - reusing", connection.Name, typehelpers.SafeString(connection.PluginInstance))
 			// store the existing connection plugin in the result map
 			requestedConnectionPluginMap[connection.Name] = existingConnectionPlugin
 			continue
@@ -139,28 +145,20 @@ func CreateConnectionPlugins(connectionNamesToCreate []string) (requestedConnect
 
 		// do we have a reattach config for this connection's plugin
 		if _, ok := getResponse.ReattachMap[connection.Name]; !ok {
-			log.Printf("[TRACE] CreateConnectionPlugins skipping connection '%s', plugin '%s' as plugin manager failed to start it", connection.Name, connection.Plugin)
+			log.Printf("[TRACE] CreateConnectionPlugins skipping connection '%s', plugin '%s' as plugin manager failed to start it", connection.Name, typehelpers.SafeString(connection.PluginInstance))
 			continue
 		}
 
 		// so we have a reattach - create a connection plugin
 		reattach := getResponse.ReattachMap[connection.Name]
-		// if this is a legacy aggregator connection, skip - we do not instantiate connectionPlugins for these
-		if connection.Type == modconfig.ConnectionTypeAggregator && !reattach.SupportedOperations.MultipleConnections {
-			log.Printf("[TRACE] %s is a legacy aggregator connection - NOT creating a ConnectionPlugin", connection.Name)
-			continue
-		}
-
 		connectionPlugin, err := createConnectionPlugin(connection, reattach)
 		if err != nil {
-			res.AddWarning(fmt.Sprintf("failed to start plugin '%s': %s", connection.PluginShortName, err))
+			res.AddWarning(fmt.Sprintf("failed to attach to plugin process for '%s': %s", typehelpers.SafeString(connection.PluginInstance), err))
 			continue
 		}
 		requestedConnectionPluginMap[connection.Name] = connectionPlugin
-		if connectionPlugin.SupportedOperations.MultipleConnections {
-			// if it supports multiple connections, store in multiConnectionPlugins too
-			multiConnectionPlugins[connection.Plugin] = connectionPlugin
-		}
+		// store in connectionPluginMap too
+		connectionPluginMap[pluginInstance] = connectionPlugin
 	}
 	log.Printf("[TRACE] all connection plugins created, populating schemas")
 
@@ -175,13 +173,52 @@ func CreateConnectionPlugins(connectionNamesToCreate []string) (requestedConnect
 	return requestedConnectionPluginMap, res
 }
 
+func handleGetFailures(getResponse *proto.GetResponse, res *RefreshConnectionResult, connectionsToCreate []*modconfig.Connection) {
+	// handle PluginSdkCompatibilityError separately
+	var pluginsWithCompatibilityError = make(map[string]struct{})
+	var compatibilityErrorConnectionCount int
+
+	for failedPluginInstance, failure := range getResponse.FailureMap {
+		// if this is a compatibility error, handle separately
+		if failure == error_helpers.PluginSdkCompatibilityError {
+			failedPluginShortName := GlobalConfig.PluginsInstances[failedPluginInstance].FriendlyName()
+			pluginsWithCompatibilityError[failedPluginShortName] = struct{}{}
+			for _, c := range GlobalConfig.Connections {
+				if typehelpers.SafeString(c.PluginInstance) == failedPluginInstance {
+					compatibilityErrorConnectionCount++
+				}
+			}
+		} else {
+			// add failures as warnings
+			res.AddWarning(fmt.Sprintf("failed to start plugin instance '%s': %s", failedPluginInstance, failure))
+		}
+
+		// figure out which connections are provided by any failed plugins
+		for _, c := range connectionsToCreate {
+			if c.Plugin == failedPluginInstance {
+
+				res.AddFailedConnection(c.Name, constants.ConnectionErrorPluginFailedToStart)
+			}
+		}
+	}
+
+	if pluginCount := len(pluginsWithCompatibilityError); pluginCount > 0 {
+		compatibilityWarning := fmt.Sprintf("failed to start %d %s using an incompatible sdk version, (required by %d %s). To update, please run: %s",
+			pluginCount,
+			utils.Pluralize("plugin", pluginCount),
+			compatibilityErrorConnectionCount,
+			utils.Pluralize("connection", compatibilityErrorConnectionCount),
+			constants.Bold(fmt.Sprintf("steampipe plugin update %s", strings.Join(maps.Keys(pluginsWithCompatibilityError), " "))))
+		res.AddWarning(compatibilityWarning)
+	}
+}
+
 // requestedConnectionPluginMap is a map of connection plugins, keyed by connection name
 // the connection names which are the keys of this map are the connections
 // which were _requested_ in the parent CreateConnectionPlugins call (i.e. not necessarily all connections)
 // NOTE: the connection plugins may provide  _more_ connections that those requested
 // - we need to populate the schema for _all_ of them
 func populateConnectionPluginSchemas(requestedConnectionPluginMap map[string]*ConnectionPlugin) error {
-
 	// build a map keyed by _all_ connection names provided by the connection plugins
 	connectionPluginMap := fullConnectionPluginMap(requestedConnectionPluginMap)
 
@@ -212,7 +249,7 @@ func populateConnectionPluginSchemas(requestedConnectionPluginMap map[string]*Co
 				continue
 			}
 
-			log.Printf("[TRACE] got schema, mode: %s", schema.Mode)
+			log.Printf("[TRACE] got schema, mode: %s, table count %d", schema.Mode, len(schema.Schema))
 			// if the schema is static, add to static schema map
 			if schema.Mode == sdkplugin.SchemaModeStatic {
 				staticSchemas[connectionPlugin.PluginName] = schema
@@ -247,53 +284,53 @@ func fullConnectionPluginMap(sparseConnectionPluginMap map[string]*ConnectionPlu
 	return connectionNameMap
 }
 
-// createConnectionPlugin sttaches to the plugin process
+// createConnectionPlugin attaches to the plugin process
 func createConnectionPlugin(connection *modconfig.Connection, reattach *proto.ReattachConfig) (*ConnectionPlugin, error) {
+	// we must have a plugin instance
+	if connection.PluginInstance == nil {
+		// unexpected
+		return nil, fmt.Errorf(fmt.Sprintf("connection '%s' has no plugin instance", connection.Name))
+	}
+
 	log.Printf("[TRACE] createConnectionPlugin for connection %s", connection.Name)
-	pluginName := connection.Plugin
+	pluginInstance := *connection.PluginInstance
 	connectionName := connection.Name
 
 	log.Printf("[TRACE] plugin manager returned reattach config for connection '%s' - pid %d",
 		connectionName, reattach.Pid)
 	if reattach.Pid == 0 {
-		log.Printf("[WARN] plugin manager returned nil PID for %s", connectionName)
-		return nil, fmt.Errorf("plugin manager returned nil PID for %s", connectionName)
+		log.Printf("[WARN] reattach config has a zero pid for connection %s", connectionName)
+		return nil, fmt.Errorf("reattach config has a zero pid for connection %s", connectionName)
 	}
 
 	// attach to the plugin process
-	pluginClient, err := attachToPlugin(reattach.Convert(), pluginName)
+	pluginClient, err := attachToPlugin(reattach.Convert(), pluginInstance)
 	if err != nil {
 		log.Printf("[TRACE] failed to attach to plugin for connection '%s' - pid %d: %s",
 			connectionName, reattach.Pid, err)
 		return nil, err
 	}
 
-	log.Printf("[TRACE] plugin client created for %s", pluginName)
+	log.Printf("[TRACE] plugin client created for %s", pluginInstance)
 
 	// now create ConnectionPlugin object return
-	connectionPlugin := NewConnectionPlugin(pluginName, pluginClient, reattach.SupportedOperations)
+	connectionPlugin := NewConnectionPlugin(connection.PluginAlias, pluginInstance, pluginClient, reattach.SupportedOperations)
 
-	// if multiple connections are NOT supported, add the config for our one and only connection
-	if reattach.SupportedOperations == nil || !reattach.SupportedOperations.MultipleConnections {
-		log.Printf("[TRACE] multiple connections NOT supported - adding single connection '%s' to ConnectionPlugin", connectionName)
-		connectionPlugin.addConnection(connectionName, connection.Config, connection.Options, connection.Type)
-	} else {
-		log.Printf("[TRACE] multiple connections ARE supported - adding all connections to ConnectionPlugin: %v", reattach.Connections)
-		// now identify all connections serviced by this plugin
-		for _, c := range reattach.Connections {
-			log.Printf("[TRACE] adding connection %s", c)
+	log.Printf("[TRACE] multiple connections ARE supported - adding all connections to ConnectionPlugin: %v", reattach.Connections)
+	// now identify all connections serviced by this plugin
+	for _, c := range reattach.Connections {
+		log.Printf("[TRACE] adding connection %s", c)
 
-			// NOTE: use GlobalConfig to access connection config
-			// we assume this has been populated either by the hub (if this is being invoked from the fdw) or the CLI
-			config, ok := GlobalConfig.Connections[c]
-			if !ok {
-				return nil, fmt.Errorf("no connection config loaded for '%s'", c)
-			}
-			connectionPlugin.addConnection(c, config.Config, config.Options, config.Type)
+		// NOTE: use GlobalConfig to access connection config
+		// we assume this has been populated either by the hub (if this is being invoked from the fdw) or the CLI
+		config, ok := GlobalConfig.Connections[c]
+		if !ok {
+			return nil, fmt.Errorf("no connection config loaded for '%s'", c)
 		}
+		connectionPlugin.addConnection(c, config.Config, config.Options, config.Type)
 	}
 
-	log.Printf("[TRACE] created connection plugin for connection: '%s', pluginName: '%s'", connectionName, pluginName)
+	log.Printf("[TRACE] created connection plugin for connection: '%s', pluginInstance: '%s'", connectionName, pluginInstance)
 	return connectionPlugin, nil
 }
 
