@@ -16,19 +16,21 @@ import (
 // json tags needed as this is stored in the connection state file
 type ConnectionState struct {
 	// the connection name
-	ConnectionName string `json:"connection,omitempty"  db:"name"`
+	ConnectionName string `json:"connection"  db:"name"`
 	// connection type (expected value: "aggregator")
 	Type *string `json:"type,omitempty"  db:"type"`
-	// should we create apostgres schema for the connection (expected values: "enable", "disable")
+	// should we create a postgres schema for the connection (expected values: "enable", "disable")
 	ImportSchema string `json:"import_schema"  db:"import_schema"`
 	// the fully qualified name of the plugin
-	Plugin string `json:"plugin,omitempty"  db:"plugin"`
+	Plugin string `json:"plugin"  db:"plugin"`
+	// the plugin instance
+	PluginInstance *string `json:"plugin_instance" db:"plugin_instance"`
 	// the connection state (pending, updating, deleting, error, ready)
-	State string `json:"state,omitempty"  db:"state"`
+	State string `json:"state"  db:"state"`
 	// error (if there is one - make a pointer to support null)
 	ConnectionError *string `json:"error,omitempty" db:"error"`
 	// schema mode - static or dynamic
-	SchemaMode string `json:"schema_mode,omitempty" db:"schema_mode"`
+	SchemaMode string `json:"schema_mode" db:"schema_mode"`
 	// the hash of the connection schema - this is used to determine if a dynamic schema has changed
 	SchemaHash string `json:"schema_hash,omitempty" db:"schema_hash"`
 	// are the comments set
@@ -38,12 +40,16 @@ type ConnectionState struct {
 	// the update time of the connection
 	ConnectionModTime time.Time `json:"connection_mod_time" db:"connection_mod_time"`
 	// the matching patterns of child connections (for aggregators)
-	Connections []string `json:"connections" db:"connections"`
+	Connections     []string `json:"connections" db:"connections"`
+	FileName        string   `json:"file_name" db:"file_name"`
+	StartLineNumber int      `json:"start_line_number" db:"start_line_number"`
+	EndLineNumber   int      `json:"end_line_number" db:"end_line_number"`
 }
 
-func NewConnectionState(remoteSchema string, connection *modconfig.Connection, creationTime time.Time) *ConnectionState {
-	return &ConnectionState{
-		Plugin:         remoteSchema,
+func NewConnectionState(connection *modconfig.Connection, creationTime time.Time) *ConnectionState {
+	state := &ConnectionState{
+		Plugin:         connection.Plugin,
+		PluginInstance: connection.PluginInstance,
 		ConnectionName: connection.Name,
 		PluginModTime:  creationTime,
 		State:          constants.ConnectionStateReady,
@@ -51,10 +57,20 @@ func NewConnectionState(remoteSchema string, connection *modconfig.Connection, c
 		ImportSchema:   connection.ImportSchema,
 		Connections:    connection.ConnectionNames,
 	}
+	state.setFilename(connection)
+	if connection.Error != nil {
+		state.SetError(connection.Error.Error())
+	}
+	return state
+}
+
+func (d *ConnectionState) setFilename(connection *modconfig.Connection) {
+	d.FileName = connection.DeclRange.Filename
+	d.StartLineNumber = connection.DeclRange.Start.Line
+	d.EndLineNumber = connection.DeclRange.End.Line
 }
 
 func (d *ConnectionState) Equals(other *ConnectionState) bool {
-
 	if d.Plugin != other.Plugin {
 		return false
 	}
@@ -76,8 +92,7 @@ func (d *ConnectionState) Equals(other *ConnectionState) bool {
 		return false
 	}
 
-	// allow for sub ms rounding errors when converting from PG
-	if d.PluginModTime.Sub(other.PluginModTime).Abs() > 1*time.Millisecond {
+	if d.pluginModTimeChanged(other) {
 		return false
 	}
 	// do not look at connection mod time as the mod time for the desired state is not relevant
@@ -85,6 +100,14 @@ func (d *ConnectionState) Equals(other *ConnectionState) bool {
 	return true
 }
 
+// allow for sub ms rounding errors when converting from PG
+func (d *ConnectionState) pluginModTimeChanged(other *ConnectionState) bool {
+	if d.PluginModTime.Sub(other.PluginModTime).Abs() > 1*time.Millisecond {
+		return true
+	}
+	return false
+}
+
 func (d *ConnectionState) CanCloneSchema() bool {
 	return d.SchemaMode != plugin.SchemaModeDynamic &&
 		d.GetType() != modconfig.ConnectionTypeAggregator
@@ -95,6 +118,7 @@ func (d *ConnectionState) Error() string {
 }
 
 func (d *ConnectionState) SetError(err string) {
+	d.State = constants.ConnectionStateError
 	d.ConnectionError = &err
 }
 
