@@ -3,44 +3,56 @@ package steampipeconfig
 import (
 	"context"
 	"fmt"
-	"github.com/jackc/pgx/v5/pgxpool"
-	typehelpers "github.com/turbot/go-kit/types"
-	"github.com/turbot/steampipe/pkg/constants"
-	"github.com/turbot/steampipe/pkg/db/db_common"
-	"golang.org/x/exp/maps"
 	"log"
 	"sort"
 	"strings"
 	"time"
 
+	"github.com/jackc/pgx/v5/pgxpool"
 	"github.com/turbot/go-kit/helpers"
 	"github.com/turbot/steampipe-plugin-sdk/v5/grpc/proto"
 	"github.com/turbot/steampipe-plugin-sdk/v5/plugin"
+	"github.com/turbot/steampipe/pkg/constants"
+	"github.com/turbot/steampipe/pkg/db/db_common"
+	pluginshared "github.com/turbot/steampipe/pkg/pluginmanager_service/grpc/shared"
 	"github.com/turbot/steampipe/pkg/steampipeconfig/modconfig"
 	"github.com/turbot/steampipe/pkg/utils"
+	"golang.org/x/exp/maps"
 )
 
 type ConnectionUpdates struct {
 	Update          ConnectionStateMap
 	Delete          map[string]struct{}
+	Error           map[string]struct{}
 	Disabled        map[string]struct{}
 	MissingComments ConnectionStateMap
-
+	// map of missing plugins, keyed by plugin ALIAS
+	// NOTE: we key by alias so the error message refers to the string which was used to specify the plugin
 	MissingPlugins map[string][]modconfig.Connection
 	// the connections which will exist after the update
 	FinalConnectionState ConnectionStateMap
 	// connection plugins required to perform the updates, keyed by connection name
-	ConnectionPlugins      map[string]*ConnectionPlugin
+	ConnectionPlugins map[string]*ConnectionPlugin
+
 	CurrentConnectionState ConnectionStateMap
 	InvalidConnections     map[string]*ValidationFailure
+	// map of plugin to connection for which we must refetch the rate limiter definitions
+	PluginsWithUpdatedBinary map[string]string
+
+	forceUpdateConnectionNames []string
+	pluginManager              pluginshared.PluginManager
 }
 
 // NewConnectionUpdates returns updates to be made to the database to sync with connection config
-func NewConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpdateConnectionNames ...string) (*ConnectionUpdates, *RefreshConnectionResult) {
-	updates, res := populateConnectionUpdates(ctx, pool, forceUpdateConnectionNames...)
+func NewConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, pluginManager pluginshared.PluginManager, opts ...ConnectionUpdatesOption) (*ConnectionUpdates, *RefreshConnectionResult) {
+	log.Println("[DEBUG] NewConnectionUpdates start")
+	defer log.Println("[DEBUG] NewConnectionUpdates end")
+
+	updates, res := populateConnectionUpdates(ctx, pool, pluginManager, opts...)
 	if res.Error != nil {
-		return updates, res
+		return nil, res
 	}
+
 	// validate the updates
 	// this will validate all plugins and connection names  and remove any updates which use invalid connections
 	updates.validate()
@@ -48,14 +60,18 @@ func NewConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpdateCo
 	return updates, res
 }
 
-func populateConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpdateConnectionNames ...string) (*ConnectionUpdates, *RefreshConnectionResult) {
-	utils.LogTime("NewConnectionUpdates start")
-	defer utils.LogTime("NewConnectionUpdates end")
-	log.Printf("[TRACE] NewConnectionUpdates")
+func populateConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, pluginManager pluginshared.PluginManager, opts ...ConnectionUpdatesOption) (*ConnectionUpdates, *RefreshConnectionResult) {
+	log.Println("[DEBUG] populateConnectionUpdates start")
+	defer log.Println("[DEBUG] populateConnectionUpdates end")
+
+	var config = &connectionUpdatesConfig{}
+	for _, opt := range opts {
+		opt(config)
+	}
 
 	conn, err := pool.Acquire(ctx)
 	if err != nil {
-		log.Printf("[WARN] failed to acquire conneciton from pool: %s", err.Error())
+		log.Printf("[WARN] failed to acquire connection from pool: %s", err.Error())
 		return nil, NewErrorRefreshConnectionResult(err)
 	}
 	defer conn.Release()
@@ -72,10 +88,10 @@ func populateConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpd
 	// build connection data for all required connections
 	// NOTE: this will NOT populate SchemaMode for the connections, as we need to load the schema for that
 	// this will be updated below on the call to updateRequiredStateWithSchemaProperties
-	requiredConnectionStateMap, missingPlugins, err := GetRequiredConnectionStateMap(GlobalConfig.Connections, currentConnectionStateMap)
-	if err != nil {
+	requiredConnectionStateMap, missingPlugins, connectionStateResult := GetRequiredConnectionStateMap(GlobalConfig.Connections, currentConnectionStateMap)
+	if connectionStateResult.Error != nil {
 		log.Printf("[WARN] failed to build required connection state: %s", err.Error())
-		return nil, NewErrorRefreshConnectionResult(err)
+		return nil, NewErrorRefreshConnectionResult(connectionStateResult.Error)
 	}
 	log.Printf("[INFO] built required connection state")
 
@@ -88,13 +104,17 @@ func populateConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpd
 	}
 
 	updates := &ConnectionUpdates{
-		Delete:               make(map[string]struct{}),
-		Disabled:             disabled,
-		Update:               ConnectionStateMap{},
-		MissingComments:      ConnectionStateMap{},
-		MissingPlugins:       missingPlugins,
-		FinalConnectionState: requiredConnectionStateMap,
-		InvalidConnections:   make(map[string]*ValidationFailure),
+		Delete:                     make(map[string]struct{}),
+		Error:                      make(map[string]struct{}),
+		Disabled:                   disabled,
+		Update:                     ConnectionStateMap{},
+		MissingComments:            ConnectionStateMap{},
+		MissingPlugins:             missingPlugins,
+		FinalConnectionState:       requiredConnectionStateMap,
+		InvalidConnections:         make(map[string]*ValidationFailure),
+		PluginsWithUpdatedBinary:   make(map[string]string),
+		forceUpdateConnectionNames: config.ForceUpdateConnectionNames,
+		pluginManager:              pluginManager,
 	}
 
 	log.Printf("[INFO] loaded connection state")
@@ -104,7 +124,7 @@ func populateConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpd
 
 	// for any connections with dynamic schema, we need to reload their schema
 	// instantiate connection plugins for all connections with dynamic schema - this will retrieve their current schema
-	dynamicSchemaHashMap, connectionsPluginsWithDynamicSchema, err := getSchemaHashesForDynamicSchemas(requiredConnectionStateMap, currentConnectionStateMap)
+	dynamicSchemaHashMap, connectionsPluginsWithDynamicSchema, err := updates.getSchemaHashesForDynamicSchemas(requiredConnectionStateMap, currentConnectionStateMap)
 	if err != nil {
 		log.Printf("[WARN] getSchemaHashesForDynamicSchemas failed: %s", err.Error())
 		return nil, NewErrorRefreshConnectionResult(err)
@@ -118,18 +138,30 @@ func populateConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpd
 	log.Printf("[INFO] identify connections to update")
 
 	modTime := time.Now()
+
 	// connections to create/update
 	for name, requiredConnectionState := range requiredConnectionStateMap {
 		// if the connection requires update, add to list
-		if connectionRequiresUpdate(forceUpdateConnectionNames, name, currentConnectionStateMap, requiredConnectionState) {
-			updates.Update[name] = requiredConnectionState
+		res := connectionRequiresUpdate(config.ForceUpdateConnectionNames, name, currentConnectionStateMap, requiredConnectionState)
+		if res.requiresUpdate {
 			log.Printf("[INFO] connection %s is out of date or missing. updates: %v", name, maps.Keys(updates.Update))
+			updates.Update[name] = requiredConnectionState
 
 			// set the connection mod time of required connection data to now
 			requiredConnectionState.ConnectionModTime = modTime
+
+			// if the plugin mod time has changed, add this to the map of connections
+			// we need to refetch the rate limiters for this plugin
+			if res.pluginBinaryChanged {
+				// store map item of plugin name to connection name (so we only have one entry per plugin)
+				pluginShortName := GlobalConfig.Connections[requiredConnectionState.ConnectionName].PluginAlias
+				updates.PluginsWithUpdatedBinary[pluginShortName] = requiredConnectionState.ConnectionName
+			}
 		}
 	}
 
+	// TODO KAI TIDY INTO FUNCTION
+
 	log.Printf("[INFO] Identify connections to delete")
 	// connections to delete - any connection which is in connection state but NOT required connections
 	for name, currentState := range currentConnectionStateMap {
@@ -140,6 +172,11 @@ func populateConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpd
 			// if required connection state is disabled and it is not currently disabled, mark for deletion
 			log.Printf("[TRACE] connection %s is disabled - marking for deletion\n", name)
 			updates.Delete[name] = struct{}{}
+		} else if updates.FinalConnectionState[name].State == constants.ConnectionStateError && currentState.State != constants.ConnectionStateError {
+			// if required connection state is disabled and it is not currently disabled, add to error map
+			// the schema will be deleted by the connection will remain in the table
+			log.Printf("[TRACE] connection %s is in error - marking for deletion\n", name)
+			updates.Error[name] = struct{}{}
 		}
 	}
 
@@ -169,6 +206,7 @@ func populateConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpd
 		connectionData, ok := currentConnectionStateMap[name]
 		// if the connection exists in the state, does the schemas hash match?
 		if ok && connectionData.SchemaHash != requiredHash {
+			log.Printf("[INFO] %s dynamic schema hash does not match - update", connectionData.ConnectionName)
 			updates.Update[name] = connectionData
 		}
 	}
@@ -187,44 +225,66 @@ func populateConnectionUpdates(ctx context.Context, pool *pgxpool.Pool, forceUpd
 	// this uses data from the ConnectionPlugins which we have now loaded
 	updates.updateRequiredStateWithSchemaProperties(dynamicSchemaHashMap)
 
-	// for all updates/deletes, if there any aggregators of the same plugin type, update those as well
+	// for all updates/deletes, if there are any aggregators of the same plugin type, update those as well
 	updates.populateAggregators()
-	res.Updates = updates
+
+	// before we return, merge in connection state warnings
+	res.AddWarning(connectionStateResult.Warnings...)
+
 	return updates, res
 }
 
-func connectionRequiresUpdate(forceUpdateConnectionNames []string, name string, currentConnectionStateMap ConnectionStateMap, requiredConnectionState *ConnectionState) bool {
+type connectionRequiresUpdateResult struct {
+	requiresUpdate      bool
+	pluginBinaryChanged bool
+}
+
+func connectionRequiresUpdate(forceUpdateConnectionNames []string, name string, currentConnectionStateMap ConnectionStateMap, requiredConnectionState *ConnectionState) connectionRequiresUpdateResult {
+	var res = connectionRequiresUpdateResult{}
+	// if the connection is in error, no update required
+	if requiredConnectionState.State == constants.ConnectionStateError {
+		return res
+	}
 	// check whether this connection exists in the state
 	currentConnectionState, schemaExistsInState := currentConnectionStateMap[name]
-
-	// if the required plugin is not installed, return false
-	if typehelpers.SafeString(requiredConnectionState.ConnectionError) == constants.ConnectionErrorPluginNotInstalled {
-		return false
-	}
 	// if the connection has been disabled, return false
 	if requiredConnectionState.Disabled() {
-		return false
+		return res
 	}
 	// is this is a new connection
 	if !schemaExistsInState {
-		return true
+		res.requiresUpdate = true
+		return res
 	}
+
+	// determine whethe the plugin mod time has changed
+	if currentConnectionState.pluginModTimeChanged(requiredConnectionState) {
+		res.requiresUpdate = true
+		res.pluginBinaryChanged = true
+		return res
+	}
+
 	// if the connection has been enabled (i.e. if it was previously DISABLED) , return true
 	if currentConnectionState.Disabled() {
-		return true
+		res.requiresUpdate = true
+		return res
 	}
 
 	// are we are forcing an update of this connection,
 	if helpers.StringSliceContains(forceUpdateConnectionNames, name) {
-		return true
+		res.requiresUpdate = true
+		return res
 	}
 
 	// has this connection previously not fully loaded
 	if currentConnectionState.State == constants.ConnectionStatePendingIncomplete {
-		return true
+		res.requiresUpdate = true
+		return res
 	}
+
 	// update if the connection state is different
-	return !currentConnectionState.Equals(requiredConnectionState)
+	res.requiresUpdate = !currentConnectionState.Equals(requiredConnectionState)
+	return res
 }
 
 // update requiredConnections - set the schema hash and schema mode for all elements of FinalConnectionState
@@ -232,9 +292,9 @@ func connectionRequiresUpdate(forceUpdateConnectionNames []string, name string,
 func (u *ConnectionUpdates) updateRequiredStateWithSchemaProperties(dynamicSchemaHashMap map[string]string) {
 	// we only need to update connections which are being updated
 	for k, v := range u.FinalConnectionState {
-		if currentConectionState, ok := u.CurrentConnectionState[k]; ok {
-			v.SchemaHash = currentConectionState.SchemaHash
-			v.SchemaMode = currentConectionState.SchemaMode
+		if currentConnectionState, ok := u.CurrentConnectionState[k]; ok {
+			v.SchemaHash = currentConnectionState.SchemaHash
+			v.SchemaMode = currentConnectionState.SchemaMode
 		}
 		// if the schemaHashMap contains this connection, use that value
 		if schemaHash, ok := dynamicSchemaHashMap[k]; ok {
@@ -243,6 +303,9 @@ func (u *ConnectionUpdates) updateRequiredStateWithSchemaProperties(dynamicSchem
 		// have we loaded a connection plugin for this connection
 		// - if so us the schema mode from the schema  it has loaded
 		if connectionPlugin, ok := u.ConnectionPlugins[k]; ok {
+			if connectionPlugin.ConnectionMap[k] == nil {
+				panic(fmt.Sprintf("reattach config for connection '%s' does not contain the config for '%s in its connection map", k, k))
+			}
 			v.SchemaMode = connectionPlugin.ConnectionMap[k].Schema.Mode
 			// if the schema mode is dynamic and the hash is not set yet, calculate the value from the connection plugin schema
 			// this will happen the first time we load a plugin - as schemaHashMap will NOT include the hash
@@ -256,16 +319,18 @@ func (u *ConnectionUpdates) updateRequiredStateWithSchemaProperties(dynamicSchem
 }
 
 func (u *ConnectionUpdates) populateConnectionPlugins(alreadyCreatedConnectionPlugins map[string]*ConnectionPlugin) *RefreshConnectionResult {
-	utils.LogTime("populateConnectionPlugins start")
-	defer utils.LogTime("populateConnectionPlugins end")
+	log.Println("[DEBUG] populateConnectionPlugins start")
+	defer log.Println("[DEBUG] populateConnectionPlugins end")
 
 	// get list of connections to update:
 	// - add connections which will be updated or have the comments updated
 	// - exclude connections already created
 	// - for any aggregator connections, instantiate the first child connection instead
+	// - if FetchRateLimitersForAllPlugins, start ALL plugins, using an abitrary exemplar connection if necessary
 	connectionsToCreate := u.getConnectionsToCreate(alreadyCreatedConnectionPlugins)
+
 	// now create them
-	connectionPlugins, res := CreateConnectionPlugins(connectionsToCreate)
+	connectionPluginsByConnection, res := CreateConnectionPlugins(u.pluginManager, connectionsToCreate)
 	// if any plugins failed to load, set those connections to error
 	for c, reason := range res.FailedConnections {
 		u.setError(c, reason)
@@ -276,10 +341,11 @@ func (u *ConnectionUpdates) populateConnectionPlugins(alreadyCreatedConnectionPl
 	}
 	// add back in the already created plugins
 	for name, connectionPlugin := range alreadyCreatedConnectionPlugins {
-		connectionPlugins[name] = connectionPlugin
+		connectionPluginsByConnection[name] = connectionPlugin
 	}
 	// and set our ConnectionPlugins property
-	u.ConnectionPlugins = connectionPlugins
+	u.ConnectionPlugins = connectionPluginsByConnection
+
 	return res
 }
 
@@ -296,13 +362,16 @@ func (u *ConnectionUpdates) getConnectionsToCreate(alreadyCreatedConnectionPlugi
 			connectionMap[child.Name] = child
 		}
 	}
+
 	// NOTE - we may have already created some connection plugins (if they have dynamic schema)
 	// - remove these from list of plugins to create
 	for name := range alreadyCreatedConnectionPlugins {
 		delete(connectionMap, name)
 	}
 
-	return maps.Keys(connectionMap)
+	connectionsToStart := maps.Keys(connectionMap)
+
+	return connectionsToStart
 }
 
 func (u *ConnectionUpdates) HasUpdates() bool {
@@ -312,24 +381,25 @@ func (u *ConnectionUpdates) HasUpdates() bool {
 func (u *ConnectionUpdates) String() string {
 	var op strings.Builder
 	update := utils.SortedMapKeys(u.Update)
-	delete := maps.Keys(u.Delete)
-	sort.Strings(delete)
+	toDelete := maps.Keys(u.Delete)
+	sort.Strings(toDelete)
 	stateConnections := utils.SortedMapKeys(u.FinalConnectionState)
 	if len(update) > 0 {
 		op.WriteString(fmt.Sprintf("Update: %s\n", strings.Join(update, ",")))
 	}
-	if len(delete) > 0 {
-		op.WriteString(fmt.Sprintf("Delete: %s\n", strings.Join(delete, ",")))
+	if len(toDelete) > 0 {
+		op.WriteString(fmt.Sprintf("Delete: %s\n", strings.Join(toDelete, ",")))
 	}
 	if len(stateConnections) > 0 {
 		op.WriteString(fmt.Sprintf("Connection state: %s\n", strings.Join(stateConnections, ",")))
 	} else {
-		op.WriteString(fmt.Sprintf("Connection state EMPTY\n"))
+		op.WriteString("Connection state EMPTY\n")
 	}
 	return op.String()
 }
 
 func (u *ConnectionUpdates) setError(connectionName string, error string) {
+	log.Printf("[INFO] ConnectionUpdates.setError connection %s: %s", connectionName, error)
 	failedConnection, ok := u.FinalConnectionState[connectionName]
 	if !ok {
 		return
@@ -344,8 +414,8 @@ func (u *ConnectionUpdates) setError(connectionName string, error string) {
 // NOTE: this mutates FinalConnectionState to set comment_set (if needed)
 func (u *ConnectionUpdates) IdentifyMissingComments() {
 	for name, state := range u.FinalConnectionState {
-		// if plugin is not installed, skip
-		if typehelpers.SafeString(state.ConnectionError) == constants.ConnectionErrorPluginNotInstalled {
+		// if the state is in error, skip
+		if state.State == constants.ConnectionStateError {
 			continue
 		}
 		if currentState, existsInCurrentState := u.CurrentConnectionState[name]; existsInCurrentState {
@@ -385,7 +455,8 @@ func (u *ConnectionUpdates) populateAggregators() {
 	log.Printf("[INFO] found %d %s with aggregators", len(pluginAggregatorMap), utils.Pluralize("plugin", len(pluginAggregatorMap)))
 
 	// for all updates/deletes, if there any aggregators of the same plugin type, update those as well
-	// build a map of all plugins with connecitons being updated/deleted
+	// build a map of all plugins with connecti
+	//ons being updated/deleted
 	modifiedPluginLookup := make(map[string]struct{})
 	for _, c := range u.Update {
 		modifiedPluginLookup[c.Plugin] = struct{}{}
@@ -407,7 +478,7 @@ func (u *ConnectionUpdates) populateAggregators() {
 
 }
 
-func getSchemaHashesForDynamicSchemas(requiredConnectionData ConnectionStateMap, connectionState ConnectionStateMap) (map[string]string, map[string]*ConnectionPlugin, error) {
+func (u *ConnectionUpdates) getSchemaHashesForDynamicSchemas(requiredConnectionData ConnectionStateMap, connectionState ConnectionStateMap) (map[string]string, map[string]*ConnectionPlugin, error) {
 	log.Printf("[TRACE] getSchemaHashesForDynamicSchemas")
 	// for every required connection, check the connection state to determine whether the schema mode is 'dynamic'
 	// if we have never loaded the connection, there will be no state, so we cannot retrieve this information
@@ -425,7 +496,7 @@ func getSchemaHashesForDynamicSchemas(requiredConnectionData ConnectionStateMap,
 			}
 		}
 	}
-	connectionsPluginsWithDynamicSchema, res := CreateConnectionPlugins(maps.Keys(connectionsWithDynamicSchema))
+	connectionsPluginsWithDynamicSchema, res := CreateConnectionPlugins(u.pluginManager, maps.Keys(connectionsWithDynamicSchema))
 	if res.Error != nil {
 		return nil, nil, res.Error
 	}
@@ -441,6 +512,10 @@ func getSchemaHashesForDynamicSchemas(requiredConnectionData ConnectionStateMap,
 	return hashMap, connectionsPluginsWithDynamicSchema, nil
 }
 
+func (u *ConnectionUpdates) GetConnectionsToDelete() []string {
+	return append(maps.Keys(u.Delete), maps.Keys(u.Error)...)
+}
+
 func pluginSchemaHash(s *proto.Schema) string {
 	var sb strings.Builder
 
