@@ -1,37 +1,43 @@
 package steampipeconfig
 
 import (
+	"bytes"
 	"fmt"
 	"log"
 	"os"
 	"path/filepath"
 	"strings"
+	"time"
 
 	"github.com/gertd/go-pluralize"
 	"github.com/hashicorp/hcl/v2"
 	filehelpers "github.com/turbot/go-kit/files"
 	"github.com/turbot/go-kit/helpers"
+	"github.com/turbot/steampipe-plugin-sdk/v5/sperr"
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/db/db_common"
 	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/filepaths"
+	"github.com/turbot/steampipe/pkg/steampipeconfig/hclhelpers"
 	"github.com/turbot/steampipe/pkg/steampipeconfig/modconfig"
 	"github.com/turbot/steampipe/pkg/steampipeconfig/options"
 	"github.com/turbot/steampipe/pkg/steampipeconfig/parse"
 	"github.com/turbot/steampipe/pkg/utils"
-	"github.com/turbot/steampipe/sperr"
 )
 
 var GlobalConfig *SteampipeConfig
 var defaultConfigFileName = "default.spc"
+var defaultConfigSampleFileName = "default.spc.sample"
 
 // LoadSteampipeConfig loads the HCL connection config and workspace options
-func LoadSteampipeConfig(modLocation string, commandName string) (*SteampipeConfig, *modconfig.ErrorAndWarnings) {
+func LoadSteampipeConfig(modLocation string, commandName string) (*SteampipeConfig, *error_helpers.ErrorAndWarnings) {
 	utils.LogTime("steampipeconfig.LoadSteampipeConfig start")
 	defer utils.LogTime("steampipeconfig.LoadSteampipeConfig end")
 
+	log.Printf("[INFO] ensureDefaultConfigFile")
+
 	if err := ensureDefaultConfigFile(filepaths.EnsureConfigDir()); err != nil {
-		return nil, modconfig.NewErrorsAndWarning(
+		return nil, error_helpers.NewErrorsAndWarning(
 			sperr.WrapWithMessage(
 				err,
 				"could not create default config",
@@ -43,14 +49,67 @@ func LoadSteampipeConfig(modLocation string, commandName string) (*SteampipeConf
 
 // LoadConnectionConfig loads the connection config but not the workspace options
 // this is called by the fdw
-func LoadConnectionConfig() (*SteampipeConfig, *modconfig.ErrorAndWarnings) {
+func LoadConnectionConfig() (*SteampipeConfig, *error_helpers.ErrorAndWarnings) {
 	return LoadSteampipeConfig("", "")
 }
 
 func ensureDefaultConfigFile(configFolder string) error {
+	// get the filepaths
 	defaultConfigFile := filepath.Join(configFolder, defaultConfigFileName)
-	if _, err := os.Stat(defaultConfigFile); os.IsNotExist(err) {
-		err = os.WriteFile(defaultConfigFile, []byte(constants.DefaultConnectionConfigContent), 0755)
+	defaultConfigSampleFile := filepath.Join(configFolder, defaultConfigSampleFileName)
+
+	// check if sample and default files exist
+	sampleExists := filehelpers.FileExists(defaultConfigSampleFile)
+	defaultExists := filehelpers.FileExists(defaultConfigFile)
+
+	var sampleContent []byte
+	var sampleModTime, defaultModTime time.Time
+
+	// if the sample file exists, load content and read mod time
+	if sampleExists {
+		sampleStat, err := os.Stat(defaultConfigSampleFile)
+		if err != nil {
+			return err
+		}
+		sampleContent, err = os.ReadFile(defaultConfigSampleFile)
+		if err != nil {
+			return err
+		}
+		sampleModTime = sampleStat.ModTime()
+	}
+
+	// if the default file exists read mod time
+	if defaultExists {
+		// get the file infos
+		defaultStat, err := os.Stat(defaultConfigFile)
+		if err != nil {
+			return err
+		}
+		// get the file mod times
+		defaultModTime = defaultStat.ModTime()
+	}
+
+	// check if the files are modified
+
+	// has the user modified the default file?
+	userModifiedDefault := defaultModTime.IsZero() ||
+		defaultModTime.After(sampleModTime) && defaultModTime.Sub(sampleModTime) > 100*time.Millisecond
+
+	// has the DefaultConnectionConfigContent been updated since the sample file was last writtne
+	sampleModified := sampleModTime.IsZero() ||
+		!bytes.Equal([]byte(constants.DefaultConnectionConfigContent), sampleContent)
+
+	// case: if sample is modified - always write new sample file content
+	if sampleModified {
+		err := os.WriteFile(defaultConfigSampleFile, []byte(constants.DefaultConnectionConfigContent), 0755)
+		if err != nil {
+			return err
+		}
+	}
+
+	// case: if sample is modified but default is not modified - write the new default file content
+	if sampleModified && !userModifiedDefault {
+		err := os.WriteFile(defaultConfigFile, []byte(constants.DefaultConnectionConfigContent), 0755)
 		if err != nil {
 			return err
 		}
@@ -58,14 +117,14 @@ func ensureDefaultConfigFile(configFolder string) error {
 	return nil
 }
 
-func loadSteampipeConfig(modLocation string, commandName string) (steampipeConfig *SteampipeConfig, errorsAndWarnings *modconfig.ErrorAndWarnings) {
+func loadSteampipeConfig(modLocation string, commandName string) (steampipeConfig *SteampipeConfig, errorsAndWarnings *error_helpers.ErrorAndWarnings) {
 	utils.LogTime("steampipeconfig.loadSteampipeConfig start")
 	defer utils.LogTime("steampipeconfig.loadSteampipeConfig end")
 
-	errorsAndWarnings = modconfig.NewErrorsAndWarning(nil)
+	errorsAndWarnings = error_helpers.NewErrorsAndWarning(nil)
 	defer func() {
 		if r := recover(); r != nil {
-			errorsAndWarnings = modconfig.NewErrorsAndWarning(helpers.ToError(r))
+			errorsAndWarnings = error_helpers.NewErrorsAndWarning(helpers.ToError(r))
 		}
 	}()
 
@@ -76,7 +135,7 @@ func loadSteampipeConfig(modLocation string, commandName string) (steampipeConfi
 	loadOptions := &loadConfigOptions{include: include}
 	if ew := loadConfig(filepaths.EnsureConfigDir(), steampipeConfig, loadOptions); ew != nil {
 		if ew.GetError() != nil {
-			return nil, ew.WrapErrorWithMessage("failed to load config")
+			return nil, ew
 		}
 		// merge the warning from this call
 		errorsAndWarnings.AddWarning(ew.Warnings...)
@@ -87,7 +146,7 @@ func loadSteampipeConfig(modLocation string, commandName string) (steampipeConfi
 	// check workspace folder exists
 	if modLocation != "" {
 		if _, err := os.Stat(modLocation); os.IsNotExist(err) {
-			return nil, modconfig.NewErrorsAndWarning(fmt.Errorf("mod location '%s' does not exist", modLocation))
+			return nil, error_helpers.NewErrorsAndWarning(fmt.Errorf("mod location '%s' does not exist", modLocation))
 		}
 
 		// only include workspace.spc from workspace directory
@@ -149,7 +208,8 @@ type loadConfigOptions struct {
 	allowedOptions []string
 }
 
-func loadConfig(configFolder string, steampipeConfig *SteampipeConfig, opts *loadConfigOptions) *modconfig.ErrorAndWarnings {
+func loadConfig(configFolder string, steampipeConfig *SteampipeConfig, opts *loadConfigOptions) *error_helpers.ErrorAndWarnings {
+	log.Printf("[INFO] loadConfig is loading connection config")
 	// get all the config files in the directory
 	configPaths, err := filehelpers.ListFiles(configFolder, &filehelpers.ListOptions{
 		Flags:   filehelpers.FilesFlat,
@@ -158,7 +218,7 @@ func loadConfig(configFolder string, steampipeConfig *SteampipeConfig, opts *loa
 
 	if err != nil {
 		log.Printf("[WARN] loadConfig: failed to get config file paths: %v\n", err)
-		return modconfig.NewErrorsAndWarning(err)
+		return error_helpers.NewErrorsAndWarning(err)
 	}
 	if len(configPaths) == 0 {
 		return nil
@@ -167,19 +227,19 @@ func loadConfig(configFolder string, steampipeConfig *SteampipeConfig, opts *loa
 	fileData, diags := parse.LoadFileData(configPaths...)
 	if diags.HasErrors() {
 		log.Printf("[WARN] loadConfig: failed to load all config files: %v\n", err)
-		return modconfig.DiagsToErrorsAndWarnings("Failed to load all config files", diags)
+		return error_helpers.DiagsToErrorsAndWarnings("Failed to load all config files", diags)
 	}
 
 	body, diags := parse.ParseHclFiles(fileData)
 	if diags.HasErrors() {
-		return modconfig.DiagsToErrorsAndWarnings("Failed to load all config files", diags)
+		return error_helpers.DiagsToErrorsAndWarnings("Failed to load all config files", diags)
 	}
 
 	// do a partial decode
 	content, moreDiags := body.Content(parse.ConfigBlockSchema)
 	if moreDiags.HasErrors() {
 		diags = append(diags, moreDiags...)
-		return modconfig.DiagsToErrorsAndWarnings("Failed to load config", diags)
+		return error_helpers.DiagsToErrorsAndWarnings("Failed to load config", diags)
 	}
 
 	// store block types which we have found in this folder - each is only allowed once
@@ -189,25 +249,38 @@ func loadConfig(configFolder string, steampipeConfig *SteampipeConfig, opts *loa
 
 	for _, block := range content.Blocks {
 		switch block.Type {
+
+		case modconfig.BlockTypePlugin:
+			plugin, moreDiags := parse.DecodePlugin(block)
+			diags = append(diags, moreDiags...)
+			if moreDiags.HasErrors() {
+				continue
+			}
+			// add plugin to steampipeConfig
+			// NOTE: this errors if there is a plugin block with a duplicate label
+			if err := steampipeConfig.addPlugin(plugin, block); err != nil {
+				return error_helpers.NewErrorsAndWarning(err)
+			}
+
 		case modconfig.BlockTypeConnection:
 			connection, moreDiags := parse.DecodeConnection(block)
 			diags = append(diags, moreDiags...)
 			if moreDiags.HasErrors() {
 				continue
 			}
-			_, alreadyThere := steampipeConfig.Connections[connection.Name]
-			if alreadyThere {
-				return modconfig.NewErrorsAndWarning(sperr.New("duplicate connection name: '%s' in '%s'", connection.Name, block.TypeRange.Filename))
+			if existingConnection, alreadyThere := steampipeConfig.Connections[connection.Name]; alreadyThere {
+				err := getDuplicateConnectionError(existingConnection, connection)
+				return error_helpers.NewErrorsAndWarning(err)
 			}
 			if ok, errorMessage := db_common.IsSchemaNameValid(connection.Name); !ok {
-				return modconfig.NewErrorsAndWarning(sperr.New("invalid connection name: '%s' in '%s'. %s ", connection.Name, block.TypeRange.Filename, errorMessage))
+				return error_helpers.NewErrorsAndWarning(sperr.New("invalid connection name: '%s' in '%s'. %s ", connection.Name, block.TypeRange.Filename, errorMessage))
 			}
 			steampipeConfig.Connections[connection.Name] = connection
 
 		case modconfig.BlockTypeOptions:
 			// check this options type is permitted based on the options passed in
 			if err := optionsBlockPermitted(block, optionBlockMap, opts); err != nil {
-				return modconfig.NewErrorsAndWarning(err)
+				return error_helpers.NewErrorsAndWarning(err)
 			}
 			opts, moreDiags := parse.DecodeOptions(block)
 			if moreDiags.HasErrors() {
@@ -229,7 +302,7 @@ func loadConfig(configFolder string, steampipeConfig *SteampipeConfig, opts *loa
 					diags = append(diags, &hcl.Diagnostic{
 						Severity: hcl.DiagWarning,
 						Summary:  warning,
-						Subject:  &block.DefRange,
+						Subject:  hclhelpers.BlockRangePointer(block),
 					})
 				}
 			}
@@ -237,10 +310,24 @@ func loadConfig(configFolder string, steampipeConfig *SteampipeConfig, opts *loa
 	}
 
 	if diags.HasErrors() {
-		return modconfig.DiagsToErrorsAndWarnings("Failed to load config", diags)
+		return error_helpers.DiagsToErrorsAndWarnings("Failed to load config", diags)
 	}
 
-	return modconfig.DiagsToErrorsAndWarnings("", diags)
+	res := error_helpers.DiagsToErrorsAndWarnings("", diags)
+
+	log.Printf("[INFO] loadConfig calling initializePlugins")
+
+	// resolve the plugins for each connection and create default plugin config
+	// for all plugins mentioned in connection config which have no explicit config
+	steampipeConfig.initializePlugins()
+
+	return res
+}
+
+func getDuplicateConnectionError(existingConnection, newConnection *modconfig.Connection) error {
+	return sperr.New("duplicate connection name: '%s'\n\t(%s:%d)\n\t(%s:%d)",
+		existingConnection.Name, existingConnection.DeclRange.Filename, existingConnection.DeclRange.Start.Line,
+		newConnection.DeclRange.Filename, newConnection.DeclRange.Start.Line)
 }
 
 func optionsBlockPermitted(block *hcl.Block, blockMap map[string]bool, opts *loadConfigOptions) error {
