@@ -2,14 +2,15 @@ package modconfig
 
 import (
 	"fmt"
-	"github.com/zclconf/go-cty/cty"
 	"sort"
 	"strings"
 
 	"github.com/hashicorp/hcl/v2"
 	"github.com/turbot/go-kit/types"
 	typehelpers "github.com/turbot/go-kit/types"
+	"github.com/turbot/steampipe/pkg/steampipeconfig/hclhelpers"
 	"github.com/turbot/steampipe/pkg/utils"
+	"github.com/zclconf/go-cty/cty"
 )
 
 // Benchmark is a struct representing the Benchmark resource
@@ -32,6 +33,23 @@ type Benchmark struct {
 	Display *string    `cty:"display" hcl:"display" json:"-"`
 }
 
+func NewRootBenchmarkWithChildren(mod *Mod, children []ModTreeItem) HclResource {
+	fullName := fmt.Sprintf("%s.%s.%s", mod.ShortName, "benchmark", "root")
+	benchmark := &Benchmark{
+		ModTreeItemImpl: ModTreeItemImpl{
+			HclResourceImpl: HclResourceImpl{
+				ShortName:       "root",
+				FullName:        fullName,
+				UnqualifiedName: fmt.Sprintf("%s.%s", "benchmark", "root"),
+				blockType:       "benchmark",
+			},
+			Mod: mod,
+		},
+	}
+	benchmark.children = append(benchmark.children, children...)
+	return benchmark
+}
+
 func NewBenchmark(block *hcl.Block, mod *Mod, shortName string) HclResource {
 	fullName := fmt.Sprintf("%s.%s.%s", mod.ShortName, block.Type, shortName)
 	benchmark := &Benchmark{
@@ -40,7 +58,7 @@ func NewBenchmark(block *hcl.Block, mod *Mod, shortName string) HclResource {
 				ShortName:       shortName,
 				FullName:        fullName,
 				UnqualifiedName: fmt.Sprintf("%s.%s", block.Type, shortName),
-				DeclRange:       block.DefRange,
+				DeclRange:       hclhelpers.BlockRange(block),
 				blockType:       block.Type,
 			},
 			Mod: mod,
