@@ -10,12 +10,14 @@ import (
 	"github.com/hashicorp/hcl/v2"
 	"github.com/turbot/go-kit/helpers"
 	"github.com/turbot/steampipe/pkg/constants"
+	"github.com/turbot/steampipe/pkg/steampipeconfig/hclhelpers"
 	"github.com/turbot/steampipe/pkg/steampipeconfig/options"
 	"github.com/turbot/steampipe/pkg/utils"
 	"golang.org/x/exp/maps"
 )
 
 const (
+	ConnectionTypePlugin     = "plugin"
 	ConnectionTypeAggregator = "aggregator"
 	ImportSchemaEnabled      = "enabled"
 	ImportSchemaDisabled     = "disabled"
@@ -30,15 +32,21 @@ var ValidImportSchemaValues = []string{ImportSchemaEnabled, ImportSchemaDisabled
 // json tags needed as this is stored in the connection state file
 type Connection struct {
 	// connection name
-	Name string `json:"name,omitempty"`
-	// name of plugin as mentioned in config
-	PluginShortName string `json:"plugin_short_name,omitempty"`
-	// fully qualified name of the plugin. derived from the short name
-	Plugin string `json:"plugin,omitempty"`
+	Name string `json:"name"`
+	// name of plugin as mentioned in config - this may be an alias to a plugin image ref
+	// OR the label of a plugin config
+	PluginAlias string `json:"plugin_short_name"`
+	// image ref plugin.
+	// we resolve this after loading all plugin configs
+	Plugin string `json:"plugin"`
+	// the label of the plugin config we are using
+	PluginInstance *string `json:"plugin_instance"`
+	// Path to the installed plugin (if it exists)
+	PluginPath *string
 	// connection type - supported values: "aggregator"
 	Type string `json:"type,omitempty"`
 	// should a schema be created for this connection - supported values: "enabled", "disabled"
-	ImportSchema string `json:"import_schema,omitempty"`
+	ImportSchema string `json:"import_schema"`
 	// list of names or wildcards which are resolved to connections
 	// (only valid for "aggregator" type)
 	ConnectionNames []string `json:"connections,omitempty"`
@@ -51,9 +59,11 @@ type Connection struct {
 	// unparsed HCL of plugin specific connection config
 	Config string `json:"config,omitempty"`
 
+	Error error
+
 	// options
 	Options   *options.Connection `json:"options,omitempty"`
-	DeclRange Range               `json:"decl_range,omitempty"`
+	DeclRange Range               `json:"decl_range"`
 }
 
 // Range represents a span of characters between two positions in a source file.
@@ -110,8 +120,10 @@ func NewPos(sourcePos hcl.Pos) Pos {
 func NewConnection(block *hcl.Block) *Connection {
 	return &Connection{
 		Name:         block.Labels[0],
-		DeclRange:    NewRange(block.TypeRange),
+		DeclRange:    NewRange(hclhelpers.BlockRange(block)),
 		ImportSchema: ImportSchemaEnabled,
+		// default to plugin
+		Type: ConnectionTypePlugin,
 	}
 }
 
@@ -145,7 +157,7 @@ func (c *Connection) SetOptions(opts options.Options, block *hcl.Block) hcl.Diag
 		diags = append(diags, &hcl.Diagnostic{
 			Severity: hcl.DiagError,
 			Summary:  fmt.Sprintf("invalid nested option type %s - only 'connection' options blocks are supported for Connections", reflect.TypeOf(o).Name()),
-			Subject:  &block.DefRange,
+			Subject:  hclhelpers.BlockRangePointer(block),
 		})
 	}
 	return diags
@@ -159,7 +171,7 @@ func (c *Connection) String() string {
 // if this is an aggregator connection, there must be at least one child, and no duplicates
 // if this is NOT an aggregator, there must be no children
 func (c *Connection) Validate(map[string]*Connection) (warnings []string, errors []string) {
-	validConnectionTypes := []string{"", ConnectionTypeAggregator}
+	validConnectionTypes := []string{ConnectionTypePlugin, ConnectionTypeAggregator}
 	if !helpers.StringSliceContains(validConnectionTypes, c.Type) {
 		return nil, []string{fmt.Sprintf("connection '%s' has invalid connection type '%s'", c.Name, c.Type)}
 	}
@@ -222,17 +234,27 @@ func (c *Connection) GetEmptyAggregatorError() string {
 		strings.Join(patterns, "','"))
 }
 
-func (c *Connection) PopulateChildren(connectionMap map[string]*Connection) {
+func (c *Connection) PopulateChildren(connectionMap map[string]*Connection) []string {
 	log.Printf("[TRACE] Connection.PopulateChildren for aggregator connection %s", c.Name)
 	c.Connections = make(map[string]*Connection)
-	for _, childName := range c.ConnectionNames {
+	var failures []string
+	for _, childPattern := range c.ConnectionNames {
 		// if this resolves as an existing connection, populate it
-		if childConnection, ok := connectionMap[childName]; ok {
-			log.Printf("[TRACE] Connection.PopulateChildren found matching connection %s", childName)
-			c.Connections[childName] = childConnection
+		if childConnection, ok := connectionMap[childPattern]; ok {
+			// verify this child connection has the same plugin instance
+			if childConnection.PluginInstance != c.PluginInstance {
+				msg := fmt.Sprintf("aggregator connection %s specifies child connection %s but it has a different plugin instance",
+					c.Name, childPattern)
+				log.Println("[WARN]", msg)
+				failures = append(failures, msg)
+			} else {
+				log.Printf("[TRACE] Connection.PopulateChildren found matching connection %s", childPattern)
+				c.Connections[childPattern] = childConnection
+			}
 			continue
 		}
-		log.Printf("[TRACE] Connection.PopulateChildren no connection matches %s - treating as a wildcard", childName)
+
+		log.Printf("[TRACE] Connection.PopulateChildren no connection matches %s - treating as a wildcard", childPattern)
 		// otherwise treat the connection name as a wildcard and see what matches
 		for name, connection := range connectionMap {
 			// if this is an aggregator connection, skip (this will also avoid us adding ourselves)
@@ -243,16 +265,17 @@ func (c *Connection) PopulateChildren(connectionMap map[string]*Connection) {
 			if _, ok := c.Connections[name]; ok {
 				continue
 			}
-			if match, _ := path.Match(childName, name); match {
-				// verify that this connection is of a compatible type
-				if connection.Plugin == c.Plugin {
+			if match, _ := path.Match(childPattern, name); match {
+				// verify that this connection is the same plugin instance
+				if connection.PluginInstance == c.PluginInstance {
 					c.Connections[name] = connection
-					log.Printf("[TRACE] connection '%s' matches pattern '%s'", name, childName)
+					log.Printf("[TRACE] connection '%s' matches pattern '%s'", name, childPattern)
 				}
 			}
 		}
 	}
 	c.ResolvedConnectionNames = maps.Keys(c.Connections)
+	return failures
 }
 
 // GetResolveConnectionNames return the names of all child connections
