@@ -3,12 +3,11 @@ package modconfig
 import (
 	"fmt"
 
-	"github.com/zclconf/go-cty/cty"
-
-	"github.com/turbot/steampipe/pkg/utils"
-
 	"github.com/hashicorp/hcl/v2"
 	typehelpers "github.com/turbot/go-kit/types"
+	"github.com/turbot/steampipe/pkg/steampipeconfig/hclhelpers"
+	"github.com/turbot/steampipe/pkg/utils"
+	"github.com/zclconf/go-cty/cty"
 )
 
 // DashboardCard is a struct representing a leaf dashboard node
@@ -42,7 +41,7 @@ func NewDashboardCard(block *hcl.Block, mod *Mod, shortName string) HclResource
 						ShortName:       shortName,
 						FullName:        fullName,
 						UnqualifiedName: fmt.Sprintf("%s.%s", block.Type, shortName),
-						DeclRange:       block.DefRange,
+						DeclRange:       hclhelpers.BlockRange(block),
 						blockType:       block.Type,
 					},
 					Mod: mod,
@@ -73,7 +72,7 @@ func (c *DashboardCard) Diff(other *DashboardCard) *DashboardTreeItemDiffs {
 	}
 
 	if !utils.SafeStringsEqual(c.Label, other.Label) {
-		res.AddPropertyDiff("Label")
+		res.AddPropertyDiff("Instance")
 	}
 
 	if !utils.SafeStringsEqual(c.Value, other.Value) {
