@@ -3,11 +3,11 @@ package modconfig
 import (
 	"fmt"
 
-	"github.com/turbot/steampipe/pkg/utils"
-	"github.com/zclconf/go-cty/cty"
-
 	"github.com/hashicorp/hcl/v2"
 	typehelpers "github.com/turbot/go-kit/types"
+	"github.com/turbot/steampipe/pkg/steampipeconfig/hclhelpers"
+	"github.com/turbot/steampipe/pkg/utils"
+	"github.com/zclconf/go-cty/cty"
 )
 
 const SnapshotQueryTableName = "custom.table.results"
@@ -40,7 +40,7 @@ func NewDashboardTable(block *hcl.Block, mod *Mod, shortName string) HclResource
 						ShortName:       shortName,
 						FullName:        fullName,
 						UnqualifiedName: fmt.Sprintf("%s.%s", block.Type, shortName),
-						DeclRange:       block.DefRange,
+						DeclRange:       hclhelpers.BlockRange(block),
 						blockType:       block.Type,
 					},
 					Mod: mod,
@@ -59,6 +59,10 @@ func NewQueryDashboardTable(qp QueryProvider) (*DashboardTable, error) {
 	if err != nil {
 		return nil, err
 	}
+	fullName, err := parsedName.ToFullName()
+	if err != nil {
+		return nil, err
+	}
 
 	c := &DashboardTable{
 		ResourceWithMetadataImpl: ResourceWithMetadataImpl{
@@ -69,7 +73,7 @@ func NewQueryDashboardTable(qp QueryProvider) (*DashboardTable, error) {
 				ModTreeItemImpl: ModTreeItemImpl{
 					HclResourceImpl: HclResourceImpl{
 						ShortName:       parsedName.Name,
-						FullName:        parsedName.ToFullName(),
+						FullName:        fullName,
 						UnqualifiedName: parsedName.ToResourceName(),
 						Title:           utils.ToStringPointer(qp.GetTitle()),
 						blockType:       BlockTypeTable,
