@@ -13,6 +13,7 @@ import (
 	filehelpers "github.com/turbot/go-kit/files"
 	typehelpers "github.com/turbot/go-kit/types"
 	"github.com/turbot/steampipe/pkg/filepaths"
+	"github.com/turbot/steampipe/pkg/steampipeconfig/hclhelpers"
 	"github.com/zclconf/go-cty/cty"
 )
 
@@ -158,7 +159,7 @@ func (m *Mod) OnDecoded(block *hcl.Block, _ ResourceMapsProvider) hcl.Diagnostic
 				return hcl.Diagnostics{&hcl.Diagnostic{
 					Severity: hcl.DiagError,
 					Summary:  "Both 'require' and legacy 'requires' blocks are defined",
-					Subject:  &block.DefRange,
+					Subject:  hclhelpers.BlockRangePointer(block),
 				}}
 			}
 		}
@@ -340,7 +341,7 @@ func (m *Mod) SetFilePath(modFilePath string) {
 }
 
 // ValidateRequirements validates that the current steampipe CLI and the installed plugins is compatible with the mod
-func (m *Mod) ValidateRequirements(pluginVersionMap map[string]*semver.Version) []error {
+func (m *Mod) ValidateRequirements(pluginVersionMap map[string]*PluginVersionString) []error {
 	validationErrors := []error{}
 	if err := m.validateSteampipeVersion(); err != nil {
 		validationErrors = append(validationErrors, err)
@@ -357,10 +358,11 @@ func (m *Mod) validateSteampipeVersion() error {
 	return m.Require.validateSteampipeVersion(m.Name())
 }
 
-func (m *Mod) validatePluginVersions(availablePlugins map[string]*semver.Version) []error {
+func (m *Mod) validatePluginVersions(availablePlugins map[string]*PluginVersionString) []error {
 	if m.Require == nil {
 		return nil
 	}
+
 	return m.Require.validatePluginVersions(m.GetInstallCacheKey(), availablePlugins)
 }
 
