@@ -7,6 +7,7 @@ import (
 	"github.com/Masterminds/semver/v3"
 	"github.com/hashicorp/hcl/v2"
 	"github.com/turbot/steampipe/pkg/ociinstaller"
+	"github.com/turbot/steampipe/pkg/steampipeconfig/hclhelpers"
 )
 
 type PluginVersion struct {
@@ -41,7 +42,7 @@ func (p *PluginVersion) String() string {
 // Initialise parses the version and name properties
 func (p *PluginVersion) Initialise(block *hcl.Block) hcl.Diagnostics {
 	var diags hcl.Diagnostics
-	p.DeclRange = block.DefRange
+	p.DeclRange = hclhelpers.BlockRange(block)
 	// handle deprecation warnings/errors
 	if p.VersionString != "" {
 		if p.MinVersionString != "" {
@@ -53,16 +54,11 @@ func (p *PluginVersion) Initialise(block *hcl.Block) hcl.Diagnostics {
 			return diags
 		}
 		// raise deprecation warning
-		// disable raising this for now, since for the v0.20.x
-		// release cycle we should support both usage (but not together)
-		//
-		// [Issue](https://github.com/turbot/steampipe/issues/3251)
-		//
-		// diags = append(diags, &hcl.Diagnostic{
-		// 	Severity: hcl.DiagWarning,
-		// 	Summary:  fmt.Sprintf("Property 'version' is deprecated - use 'min_version' instead, in plugin '%s' require block", p.RawName),
-		// 	Subject:  &p.DeclRange,
-		// })
+		diags = append(diags, &hcl.Diagnostic{
+			Severity: hcl.DiagWarning,
+			Summary:  fmt.Sprintf("Property 'version' is deprecated - use 'min_version' instead, in plugin '%s' require block", p.RawName),
+			Subject:  &p.DeclRange,
+		})
 		// copy into new property
 		p.MinVersionString = p.VersionString
 	}
