@@ -6,6 +6,7 @@ import (
 	"github.com/hashicorp/hcl/v2/gohcl"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
 	"github.com/turbot/go-kit/helpers"
+	"github.com/turbot/steampipe/pkg/steampipeconfig/hclhelpers"
 	"github.com/turbot/steampipe/pkg/steampipeconfig/modconfig"
 	"github.com/turbot/steampipe/pkg/steampipeconfig/modconfig/var_config"
 )
@@ -232,7 +233,7 @@ func resourceForBlock(block *hcl.Block, parseCtx *ModParseContext) (modconfig.Hc
 		return nil, hcl.Diagnostics{&hcl.Diagnostic{
 			Severity: hcl.DiagError,
 			Summary:  fmt.Sprintf("resourceForBlock called for unsupported block type %s", block.Type),
-			Subject:  &block.DefRange,
+			Subject:  hclhelpers.BlockRangePointer(block),
 		},
 		}
 	}
@@ -291,9 +292,6 @@ func decodeVariable(block *hcl.Block, parseCtx *ModParseContext) (*modconfig.Var
 
 func decodeQueryProvider(block *hcl.Block, parseCtx *ModParseContext) (modconfig.QueryProvider, *DecodeResult) {
 	res := newDecodeResult()
-
-	// TODO  [node_reuse] need raise errors for invalid properties https://github.com/turbot/steampipe/issues/2923
-
 	// get shell resource
 	resource, diags := resourceForBlock(block, parseCtx)
 	res.handleDecodeDiags(diags)
@@ -361,8 +359,6 @@ func decodeQueryProviderBlocks(block *hcl.Block, content *hclsyntax.Body, resour
 func decodeNodeAndEdgeProvider(block *hcl.Block, parseCtx *ModParseContext) (modconfig.HclResource, *DecodeResult) {
 	res := newDecodeResult()
 
-	// TODO  [node_reuse] need raise errors for invalid properties https://github.com/turbot/steampipe/issues/2923
-
 	// get shell resource
 	resource, diags := resourceForBlock(block, parseCtx)
 	res.handleDecodeDiags(diags)
@@ -683,8 +679,7 @@ func handleModDecodeResult(resource modconfig.HclResource, res *DecodeResult, bl
 
 	// if resource supports metadata, save it
 	if resourceWithMetadata, ok := resource.(modconfig.ResourceWithMetadata); ok {
-		body := block.Body.(*hclsyntax.Body)
-		moreDiags = addResourceMetadata(resourceWithMetadata, body.SrcRange, parseCtx)
+		moreDiags = addResourceMetadata(resourceWithMetadata, resource.GetHclResourceImpl().DeclRange, parseCtx)
 		res.addDiags(moreDiags)
 	}
 }
